/****************************************************************
*
*    Proprietary Data of Adapitve Networks Inc.
*              All rights reserved
*
* Notice:
*		This file contains trade secret data which
*		is the property of Adpative Networks Inc.  This
*		file may not be used for any purpose unless
*		permission is granted in writing by an officier
*		of Adapitve Networks Inc.
*
*
* Filename
*		DSP1000.h
*
* Description:
*		header file for functionality flags
*
* Revision History:
*	J. Goh 		Nov. 08th 2005		Original
*	Khaled Saab	August 17th 2011	moved the version numbers to version.h
****************************************************************/
#pragma once
#ifndef _DSP_1000_H__
#define _DSP_1000_H__

#ifdef __cplusplus
extern "C"
{
#endif

//-------------------------------------------------------------------------
// Include Standard C Language Header Files
//-------------------------------------------------------------------------
#include <stdint.h>

#ifdef __TI_COMPILER_VERSION__
#include <std.h>
#include <hwi.h>
#endif
#ifdef CORTEX_APP
#include <rtl.h>		//os_mut_xxx
#include <inc/hw_nvic.h>
#include <inc/hw_ints.h>
#endif

//-------------------------------------------------------------------------
// Include any other Header Files
//-------------------------------------------------------------------------
#include "../Bin/version.h" 		//version numbers
#include "../Shared/targets.h"		//targets select

//-------------------------------------------------------------------------
// Compile flags
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
// Global defines
//-------------------------------------------------------------------------
#ifndef TRUE
#	define TRUE  				(1==1)				//0x01
#	define FALSE 				(!TRUE)				//0x00
#endif

#ifndef SUCCESS
#	define SUCCESS 				(1==1)				//0x01
#	define FAIL 				!SUCCESS			//0x00
#	define PASS  				SUCCESS				//0x01
#endif
#ifndef BUSY
#	define BUSY 				2					//0x02
#endif
#ifndef DONE
#	define DONE					3					//0x03
#endif

//generic loops max poll count
#ifndef MAX_POLL_COUNT
	#define MAX_POLL_COUNT		100
#endif

#ifndef PI
#  define PI 3.14159265358979323846
#endif

#ifndef IMAGE_CONSTRUCTION_GUI
#define STATIC static
#else
#define STATIC
#endif

//define for number of bits in each word
#define WORD_SIZE_M_01_BITS		-1		//bit
#define WORD_SIZE_00_BITS		 0		//bit
#define WORD_SIZE_01_BITS		 1		//bit
#define WORD_SIZE_02_BITS		 2
#define WORD_SIZE_03_BITS		 3
#define WORD_SIZE_04_BITS		 4		//nibbel
#define WORD_SIZE_05_BITS		 5
#define WORD_SIZE_06_BITS		 6
#define WORD_SIZE_07_BITS		 7
#define WORD_SIZE_08_BITS		 8		//byte
#define WORD_SIZE_09_BITS		 9
#define WORD_SIZE_10_BITS		10
#define WORD_SIZE_12_BITS		12		//Golay (un-encoded)
#define WORD_SIZE_16_BITS		16		//int 16
#define WORD_SIZE_24_BITS		24		//Golay (encoded)
#define WORD_SIZE_32_BITS		32		//int 32
#define WORD_SIZE_64_BITS		64		//int 64

#define WORD_MASK_01_BITS		0x0000000000000001	//bit
#define WORD_MASK_02_BITS		0x0000000000000003
#define WORD_MASK_03_BITS		0x0000000000000007
#define WORD_MASK_04_BITS		0x000000000000000F	//nibbel
#define WORD_MASK_05_BITS		0x000000000000001F
#define WORD_MASK_06_BITS		0x000000000000003F
#define WORD_MASK_07_BITS		0x000000000000007F
#define WORD_MASK_08_BITS		0x00000000000000FF	//byte
#define WORD_MASK_09_BITS		0x00000000000001FF
#define WORD_MASK_10_BITS		0x00000000000003FF
#define WORD_MASK_12_BITS		0x0000000000000FFF	//Golay (un=-encoded)
#define WORD_MASK_16_BITS		0x000000000000FFFF	//int 16
#define WORD_MASK_24_BITS		0x0000000000FFFFFF	//Golay (encoded)
#define WORD_MASK_32_BITS		0x00000000FFFFFFFF	//int 32
#define WORD_MASK_64_BITS		0xFFFFFFFFFFFFFFFF	//int 64

//defines for operatingMode variable
#define NO_OPERATING_MODE		0x00
#define	TEST_MODE				0x01
#define AN192_MODE				0x02
#define	AN1000_MODE				0x04
#define FLASH_SERIAL_MODE		0x08
#define	FLASH_POWERLINE_MODE	0x10

//defines to identify the different cores
//Hot bit coding is used
#define DSP_CORE0				0x01
#define DSP_CORE1				0x02
#define DSP_CORE2				0x04
#define DSP_CORE3				0x08
#define DSP_CORE4				0x10
#define DSP_CORE5				0x20
#define CORTEX_CORE				0x40
#define EEPROM_INTERFACE		0x80
#define ALL_CORES				0x7F	//DSP_CORE0 | DSP_CORE1 | ...
#define ALL_DSP				    0x3F	//DSP_CORE0 | DSP_CORE1 | ...
#define NUM_CORES				0x07	//number of cores in the system

#if defined (CORTEX_APP)
	//Note: IntMasterDisable(), IntMasterEnable() are wrappers that
	//		manage the access to PRIMASK.
	//		PRIMASK is only accessible in privileged mode. The MSR (Move
	//		from special register to general register) and MRS (Move from
	//		general register to special register) instructions are used
	//		to access the PRIMASK register, and the CPS (Change processor
	//		state) instructions (CPSID, CPSIE) may be used to change the
	//		value of the PRIMASK register.

	//Those defines are used by the Cortex ISR or tasks to protect
	//the critical code.
	#define HWI_MasterVariable		uint32_t cpsiState;	//uint32_t
	#define HWI_MasterDisable()		cpsiState = IntMasterDisable( );
	#define HWI_MasterEnable()		if(cpsiState == false) IntMasterEnable( );

	//Those defines are used by the Cortex tasks to protect the critical
	//code
	//Note: on the Cortex, this code can't be called from an ISR context
	#define MUTEX_TIMEOUT			100
	#define MUT_Init()				os_mut_init(&gMutexFlag);
	#define MUT_Wait()				os_mut_wait(&gMutexFlag, MUTEX_TIMEOUT);
	#define MUT_Release()			os_mut_release(&gMutexFlag);

	//Those defines are used by both the Cortex and the DSP to protect
	//critical code. On the Cortex side, Mutex is used to elevate the
	//task priority, while on the DSP, Hwi disable is used.
	//Note: on the Cortex, this code can't be called from an ISR context
	#define VAR_CRITICAL			{ /*empty */ }
	#define ENTER_CRITICAL() 		MUT_Wait();
	#define EXIT_CRITICAL()			MUT_Release();
// 	#define VAR_CRITICAL			HWI_MasterVariable
// 	#define ENTER_CRITICAL() 		HWI_MasterDisable(); MUT_Wait();
// 	#define EXIT_CRITICAL()			MUT_Release();	HWI_MasterEnable();

#elif defined ( __TI_COMPILER_VERSION__)
	//Those defines are used by the DSP to protect critical code.
	#define HWI_MasterVariable		uint32_t key;		//uint_32
	#define HWI_MasterDisable()		key = HWI_disable();
	#define HWI_MasterEnable()		HWI_restore(key);

	//Those defines are used by both the Cortex and the DSP to protect
	//critical code. On the Cortex side, Mutex is used to elevate the
	//task priority, while on the DSP, Hwi disable is used.
	//Note: on the Cortex, this code can't be called from an ISR context
 	#define VAR_CRITICAL			uint32_t key;
 	#define ENTER_CRITICAL() 		key = HWI_disable();
 	#define EXIT_CRITICAL()			HWI_restore(key);

	//empty defines for compatibility
	#define MUT_Init() 				{ /*empty */ }
	#define MUT_Wait() 				{ /*empty */ }
	#define MUT_Release() 			{ /*empty */ }

#else
	#define HWI_MasterVariable		{ /*empty */ }
	#define HWI_MasterDisable()		{ /*empty */ }
	#define HWI_MasterEnable()		{ /*empty */ }

	#define VAR_CRITICAL			{ /*empty */ }
 	#define ENTER_CRITICAL() 		{ /*empty */ }
 	#define EXIT_CRITICAL()			{ /*empty */ }

	//empty defines for compatibility
	#define MUT_Init() 				{ /*empty */ }
	#define MUT_Wait() 				{ /*empty */ }
	#define MUT_Release() 			{ /*empty */ }

#endif //define for Hwi enable/disable



//C6472 MegaModule revision ID
#if defined ( __TI_COMPILER_VERSION__)
#define MM_REVID_REG 0x01812000u
//#define MM_REVID_REG 0x02A8070Cu
#endif


//-------------------------------------------------------------------------
// Global varialbes redeclaration
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
// System Function Prototypes
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
// System Function Prototypes redeclaration
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
// Global symbols defined in the linker command file
//-------------------------------------------------------------------------
#ifdef __cplusplus
}
#endif

#endif //_DSP1000_H__
/*** end of file *****************************************************/
