/****************************************************************
 *
 *    Proprietary Data of Adaptive Networks Inc.
 *              All rights reserved
 *
 * Notice:
 *		This file contains trade secret data which
 *		is the property of Adaptive Networks Inc.  This
 *		file may not be used for any purpose unless
 *		permission is granted in writing by an officer
 *		of Adaptive Networks Inc.
 *
 *
 * Filename
 *		MacLink.h
 *
 * Description:
 *
 *
 * Author:
 *		khaled saab		original version
 *
 * Revision History:
 * khaled saab		original version
 *	- October 20th 2009	creation date
 * Mustapha Ait Yakoub
 *	- June 7th 2011	Revised version: DATA_HEADER64 was added
 *
 ****************************************************************/
#ifndef __MAC_LINK_H__
#define __MAC_LINK_H__

#include <stdint.h>
#include "../Shared/targets.h"
#include "../Shared/dataAPI.h"

// Commands used to tag the different packet
#define CMD_DEFAULT				0x00
#define CMD_SEND_PACKET			0x01 	//packet to send over the power line to Ethernet host
#define CMD_RECEIVED_PACKET		0x02	//packet received from power line, send to Ethernet host
#define CMD_REPLY_PACKET		0x02	//send to host (similar to RECEIVED PACKET, duplicate for clarity)
#define CMD_SEND_ENDPOINT		0x03	//packet to send over the power line to EndPoint host


typedef struct endPointHeader_t {
	uint8_t  srcEndPoint;	//Ethernet, RS232, PowerLine ... (see interfaces.h)
    uint8_t  dstEndPoint;   //Ethernet, RS232, PowerLine ... (see interfaces.h)
	uint32_t addrEndPoint;	//address to read or to write to

	uint16_t command;		//command:
							//	- CMD_SEND_PACKET
							//	- CMD_SNMP_PACKET
							//Note: This is REDUNDANT to the packetDstEP()!
							//		There is no point in DOUBLE tagging
							//		each packet. Most likely the packetCmd()
							//		could be removed.
}ENDPOINT_HEADER;

typedef struct packetStatus_t {
    uint16_t state;         //PACKET_IN_USE_STATE, PACKET_READY_STATE ...
	void    *queue;			//pointer to trace back to the head QUEUE_HEADER.
    void    *next;          //pointer to the next structure (chain list of packets)
} PACKET_STATUS;

typedef struct packetHeader_t {
    //information used to process the packet
    //this information is used by the packetAPI, queueAPI,
    //the ENDPOINTs and the task manager.
    PACKET_STATUS   status; // packet status

    //the packet payload
    DATA_HEADER_  data;      //packet payload (assembled)

    //End point Interface (Input/Output)
	ENDPOINT_HEADER endPoint;   //the data source interface
}PACKET_HEADER;

#endif
