/****************************************************************
*
*    Proprietary Data of Adapitve Networks Inc.
*              All rights reserved
*
* Notice:
*		This file contains trade secret data which
*		is the property of Adpative Networks Inc.  This
*		file may not be used for any purpose unless
*		permission is granted in writing by an officier
*		of Adapitve Networks Inc.
*
*
* Filename
*		dataAPI.h
*
* Description:
*		header file data header, packet header...
*
* Author:
*		Khaled Saab		original version
*
* Revision History:
* Khaled Saab
*	- Jan 1th 2010	Original
* Mustapha Ait Yakoub
*	- June 7th  2011 functions which refer to 64 bits were added
*
****************************************************************/
#ifndef __DATAAPI_H
#define __DATAAPI_H

//-------------------------------------------------------------------------
// Include header Files
//-------------------------------------------------------------------------
#include <stdint.h>
//#include "../Shared/MacLink.h"

//We need a way to have the dataHeader support 64 bits, 32 bits and 16bits !!!!
//This way we could use same function calls to read/write/put/get/reset...
//data received/transmitted from decoder
#define DEFAULT_DATA_HEADER {NULL, NULL, 0, 0, 0, 0, 0, 0}
typedef struct dataHeader64_t {
	uint64_t  *ptr;			//pointer to the packet (heap)
	uint64_t  *ptrSave;		//make a copy of the pointer
	uint16_t   format;		//bits, bytes, words, complex, mantissa...
	 int16_t   putIndex;	//the current write index
	 int16_t   getIndex;	//the current read index
	 int16_t   RWIndex;		//read/write index for user
	 int16_t   length;		//number of words
	 int16_t   maxLength;	//maximal size of the buffer
}DATA_HEADER64;

typedef struct dataHeader32_t {
	uint32_t  *ptr;			//pointer to the packet (heap)
	uint32_t  *ptrSave;		//make a copy of the pointer
	uint16_t   format;		//bits, bytes, words, complex, mantissa...
	 int16_t   putIndex;	//the current write index
	 int16_t   getIndex;	//the current read index
	 int16_t   RWIndex;		//read/write index for user
	 int16_t   length;		//number of words
	 int16_t   maxLength;	//maximal size of the buffer
}DATA_HEADER32;

//data received/transmitted from/to power line
typedef struct dataHeader16_t {
	uint16_t  *ptr;			//pointer to the packet (heap)
	uint16_t  *ptrSave;		//make a copy of the pointer
	uint16_t   format;		//bits (this is the only format supported)
	 int16_t   putIndex;	//the current write index
	 int16_t   getIndex;	//the current read index
	 int16_t   RWIndex;		//read/write index for user
	 int16_t   length;		//number of words
	 int16_t   maxLength;	//maximal size of the buffer
}DATA_HEADER16;

//internal data structer used for segmentation and re-assembly
typedef struct dataHeader08_t {
	uint8_t  *ptr;			//pointer to the packet (heap)
	uint8_t  *ptrSave;		//make a copy of the pointer
	uint16_t  format;		//bits (this is the only format supported)
	 int16_t  putIndex;		//the current write index
	 int16_t  getIndex;		//the current read index
	 int16_t  RWIndex;		//read/write index for user
	 int16_t  length;		//number of words
	 int16_t  maxLength;	//maximal size of the buffer
}DATA_HEADER08;

#define DATA_HEADER01     DATA_HEADER08     //serial format
#define DATA_HEADER_      DATA_HEADER08     //should correspond to the
                                            //#define declared in
                                            //dataAPI.h

//-------------------------------------------------------------------------
// Global defines
//-------------------------------------------------------------------------
//used to align the data to a specific byte boundary
#define BOUNDARY_OF_1_BYTE      1
#define BOUNDARY_OF_2_BYTE      2
#define BOUNDARY_OF_3_BYTE      3
#define BOUNDARY_OF_4_BYTE      4
#define BOUNDARY_OF_5_BYTE      5
#define BOUNDARY_OF_6_BYTE      6
#define BOUNDARY_OF_7_BYTE      7
#define BOUNDARY_OF_8_BYTE      8

//data access
#define dataUsed01(data_p)					((((data_p)->putIndex > (data_p)->getIndex)) ? ((data_p)->putIndex - (data_p)->getIndex) : 0)
#define dataFree01(data_p)					((((data_p)->length   > (data_p)->putIndex)) ? ((data_p)->length   - (data_p)->putIndex) : 0)

#define dataSetFormat01(data_p, val)		((data_p)->format     = val)
#define dataSetMaxLength01(data_p, val)		((data_p)->maxLength  = val)
#define dataFormat01(data_p)				((data_p)->format)
#define dataLength01(data_p)				((data_p)->length)
#define dataMaxLength01(data_p)				((data_p)->maxLength)
#define dataGetIndex01(data_p)				((data_p)->getIndex)
#define dataPutIndex01(data_p)				((data_p)->putIndex)
#define dataRWIndex01(data_p)               ((data_p)->RWIndex)

#define dataPtr01(data_p, index)			(&((data_p)->ptr[index]))
#define dataGetPtr01(data_p)				((data_p)->ptr)
#define dataSetPtr01(data_p, Ptr)			((data_p)->ptr = Ptr)

#define dataSetPtr01(data_p, Ptr)			((data_p)->ptr = Ptr)
#define dataResetLength01(data_p)			((data_p)->length     = 0)
#define dataResetFormat01(data_p)			((data_p)->format     = WORD_SIZE_01_BITS)
#define dataResetGetIndex01(data_p) 		((data_p)->getIndex   = 0)
#define dataResetPutIndex01(data_p) 		((data_p)->putIndex   = 0)
#define dataResetRWIndex01(data_p)          ((data_p)->RWIndex    = 0)

#ifdef _DEBUG
void dataSetLength01(DATA_HEADER01 *data_p, uint16_t val);
void dataSetGetIndex01(DATA_HEADER01 *data_p, uint16_t val);
void dataSetPutIndex01(DATA_HEADER01 *data_p, uint16_t val);
void dataSetRWIndex01(DATA_HEADER01 *data_p, uint16_t val);
#else
#define dataSetLength01(data_p, val)		((data_p)->length     = val)
#define dataSetGetIndex01(data_p, val)		((data_p)->getIndex   = val)
#define dataSetPutIndex01(data_p, val)		((data_p)->putIndex   = val)
#define dataSetRWIndex01(data_p, val)       ((data_p)->RWIndex    = val)
#endif


//data access
#define dataUsed08(data_p)					((((data_p)->putIndex > (data_p)->getIndex)) ? ((data_p)->putIndex - (data_p)->getIndex) : 0)
#define dataFree08(data_p)					((((data_p)->length   > (data_p)->putIndex)) ? ((data_p)->length   - (data_p)->putIndex) : 0)

#define dataSetFormat08(data_p, val)		((data_p)->format     = val)
#define dataSetMaxLength08(data_p, val)		((data_p)->maxLength  = val)
#define dataFormat08(data_p)				((data_p)->format)
#define dataLength08(data_p)				((data_p)->length)
#define dataGetIndex08(data_p)				((data_p)->getIndex)
#define dataPutIndex08(data_p)				((data_p)->putIndex)
#define dataRWIndex08(data_p)               ((data_p)->RWIndex)

#define dataMaxLength08(data_p)				((data_p)->maxLength)
#define dataSetMaxLength08(data_p ,val)		((data_p)->maxLength = val)
#define dataGetMaxLength08(data_p)			((data_p)->maxLength)

#define dataPtr08(data_p, index)			(&((data_p)->ptr[index]))
#define dataGetPtr08(data_p)				((data_p)->ptr)
#define dataSetPtr08(data_p, Ptr)			((data_p)->ptr = Ptr)

#define dataResetLength08(data_p)			((data_p)->length     = 0)
#define dataResetFormat08(data_p)			((data_p)->format     = WORD_SIZE_08_BITS)
#define dataResetGetIndex08(data_p) 		((data_p)->getIndex   = 0)
#define dataResetPutIndex08(data_p) 		((data_p)->putIndex   = 0)
#define dataResetRWIndex08(data_p)          ((data_p)->RWIndex    = 0)

#ifdef _DEBUG
void    dataWrite08(DATA_HEADER08 *data_p, uint8_t val, int16_t index);
uint8_t dataRead08 (DATA_HEADER08 *data_p, int16_t index);
void    dataPut08  (DATA_HEADER08 *data_p, uint8_t val);
uint8_t dataGet08  (DATA_HEADER08 *data_p);
void    dataSetLength08(DATA_HEADER08 *data_p, uint16_t val);
void    dataSetGetIndex08(DATA_HEADER08 *data_p, uint16_t val);
void    dataSetPutIndex08(DATA_HEADER08 *data_p, uint16_t val);
void    dataSetRWIndex08(DATA_HEADER08 *data_p, uint16_t val);
#else
#define dataWrite08(data_p, val, index)		((data_p)->ptr[index] = val)
#define dataRead08(data_p,index)			((data_p)->ptr[index])
#define dataPut08(data_p,val)				((data_p)->ptr[(data_p)->putIndex++] = val)
#define dataGet08(data_p)					((data_p)->ptr[(data_p)->getIndex++])
#define dataSetLength08(data_p, val)		((data_p)->length     = val)
#define dataSetGetIndex08(data_p, val)		((data_p)->getIndex   = val)
#define dataSetPutIndex08(data_p, val)		((data_p)->putIndex   = val)
#define dataSetRWIndex08(data_p, val)       ((data_p)->RWIndex    = val)
#endif

//data access
#define dataUsed16(data_p)					((((data_p)->putIndex > (data_p)->getIndex)) ? ((data_p)->putIndex - (data_p)->getIndex) : 0)
#define dataFree16(data_p)					((((data_p)->length   > (data_p)->putIndex)) ? ((data_p)->length   - (data_p)->putIndex) : 0)

#define dataSetFormat16(data_p, val)		((data_p)->format     = val)
#define dataSetMaxLength16(data_p, val)		((data_p)->maxLength  = val)
#define dataFormat16(data_p)				((data_p)->format)
#define dataLength16(data_p)				((data_p)->length)
#define dataGetIndex16(data_p)				((data_p)->getIndex)
#define dataPutIndex16(data_p)				((data_p)->putIndex)
#define dataRWIndex16(data_p)               ((data_p)->RWIndex)

#define dataMaxLength16(data_p)				((data_p)->maxLength)
#define dataSetMaxLength16(data_p ,val)		((data_p)->maxLength = val)
#define dataGetMaxLength16(data_p)			((data_p)->maxLength)

#define dataPtr16(data_p, index) 		    (uint16_t *) (&((data_p)->ptr[index]))
#define dataGetPtr16(data_p)				((data_p)->ptr)
#define dataSetPtr16(data_p, Ptr)			((data_p)->ptr = Ptr)

#define dataResetLength16(data_p)			((data_p)->length     = 0)
#define dataResetFormat16(data_p)			((data_p)->format     = WORD_SIZE_16_BITS)
#define dataResetGetIndex16(data_p) 		((data_p)->getIndex   = 0)
#define dataResetPutIndex16(data_p) 		((data_p)->putIndex   = 0)
#define dataResetRWIndex16(data_p)          ((data_p)->RWIndex    = 0)

#ifdef _DEBUG
void     dataWrite16(DATA_HEADER16 *data_p, uint16_t val, int16_t index);
uint16_t dataRead16 (DATA_HEADER16 *data_p, int16_t index);
void     dataPut16  (DATA_HEADER16 *data_p, uint16_t val);
uint16_t dataGet16  (DATA_HEADER16 *data_p);
void     dataSetLength16(DATA_HEADER16 *data_p, uint16_t val);
void     dataSetGetIndex16(DATA_HEADER16 *data_p, uint16_t val);
void     dataSetPutIndex16(DATA_HEADER16 *data_p, uint16_t val);
void     dataSetRWIndex16(DATA_HEADER16 *data_p, uint16_t val);
#else
#define dataWrite16(data_p, val, index)		((data_p)->ptr[index] = val)
#define dataRead16(data_p,index)			((data_p)->ptr[index])
#define dataPut16(data_p,val)				((data_p)->ptr[(data_p)->putIndex++] = val)
#define dataGet16(data_p)					((data_p)->ptr[(data_p)->getIndex++])
#define dataSetLength16(data_p, val)		((data_p)->length     = val)
#define dataSetGetIndex16(data_p, val)		((data_p)->getIndex   = val)
#define dataSetPutIndex16(data_p, val)		((data_p)->putIndex   = val)
#define dataSetRWIndex16(data_p, val)       ((data_p)->RWIndex    = val)
#endif


//data access
#define dataUsed32(data_p)					((((data_p)->putIndex > (data_p)->getIndex)) ? ((data_p)->putIndex - (data_p)->getIndex) : 0)
#define dataFree32(data_p)					((((data_p)->length   > (data_p)->putIndex)) ? ((data_p)->length   - (data_p)->putIndex) : 0)

#define dataSetFormat32(data_p, val)		((data_p)->format     = val)
#define dataSetMaxLength32(data_p, val)		((data_p)->maxLength  = val)
#define dataFormat32(data_p)				((data_p)->format)
#define dataLength32(data_p)				((data_p)->length)
#define dataGetIndex32(data_p)				((data_p)->getIndex)
#define dataPutIndex32(data_p)				((data_p)->putIndex)
#define dataRWIndex32(data_p)               ((data_p)->RWIndex)

#define dataMaxLength32(data_p)				((data_p)->maxLength)
#define dataSetMaxLength32(data_p ,val)		((data_p)->maxLength = val)
#define dataGetMaxLength32(data_p)			((data_p)->maxLength)

#define dataPtr32(data_p, index)			(&((data_p)->ptr[index]))
#define dataGetPtr32(data_p)				((data_p)->ptr)
#define dataSetPtr32(data_p, Ptr)			((data_p)->ptr = Ptr)

#define dataResetLength32(data_p)			((data_p)->length     = 0)
#define dataResetFormat32(data_p)			((data_p)->format     = WORD_SIZE_32_BITS)
#define dataResetGetIndex32(data_p) 		((data_p)->getIndex   = 0)
#define dataResetPutIndex32(data_p) 		((data_p)->putIndex   = 0)
#define dataResetRWIndex32(data_p)          ((data_p)->RWIndex    = 0)


#ifdef _DEBUG
void     dataWrite32(DATA_HEADER32 *data_p, uint32_t val, int16_t index);
uint32_t dataRead32 (DATA_HEADER32 *data_p, int16_t index);
void     dataPut32  (DATA_HEADER32 *data_p, uint32_t val);
uint32_t dataGet32  (DATA_HEADER32 *data_p);
void     dataSetLength32(DATA_HEADER32 *data_p, uint16_t val);
void     dataSetGetIndex32(DATA_HEADER32 *data_p, uint16_t val);
void     dataSetPutIndex32(DATA_HEADER32 *data_p, uint16_t val);
void     dataSetRWIndex32(DATA_HEADER32 *data_p, uint16_t val);
#else
#define dataWrite32(data_p, val, index)		((data_p)->ptr[index] = val)
#define dataRead32(data_p,index)			((data_p)->ptr[index])
#define dataPut32(data_p,val)				((data_p)->ptr[(data_p)->putIndex++] = val)
#define dataGet32(data_p)					((data_p)->ptr[(data_p)->getIndex++])
#define dataSetLength32(data_p, val)		((data_p)->length     = val)
#define dataSetGetIndex32(data_p, val)		((data_p)->getIndex   = val)
#define dataSetPutIndex32(data_p, val)		((data_p)->putIndex   = val)
#define dataSetRWIndex32(data_p, val)       ((data_p)->RWIndex    = val)
#endif

//------------------------------------------------------------------------------
// for need for calculating the Euclidean distance those functions that refer
// to 64 bits are added, namely: DATA_HEADER64, dataWrite64, dataRead64 ...
//------------------------------------------------------------------------------
//data access
#define dataUsed64(data_p)					((((data_p)->putIndex > (data_p)->getIndex)) ? ((data_p)->putIndex - (data_p)->getIndex) : 0)
#define dataFree64(data_p)					((((data_p)->length   > (data_p)->putIndex)) ? ((data_p)->length   - (data_p)->putIndex) : 0)

#define dataSetFormat64(data_p, val)		((data_p)->format     = val)
#define dataSetMaxLength64(data_p, val)		((data_p)->maxLength  = val)
#define dataFormat64(data_p)				((data_p)->format)
#define dataLength64(data_p)				((data_p)->length)
#define dataGetIndex64(data_p)				((data_p)->getIndex)
#define dataPutIndex64(data_p)				((data_p)->putIndex)
#define dataRWIndex64(data_p)               ((data_p)->RWIndex)

#define dataMaxLength64(data_p)				((data_p)->maxLength)
#define dataSetMaxLength64(data_p ,val)		((data_p)->maxLength = val)
#define dataGetMaxLength64(data_p)			((data_p)->maxLength)

#define dataPtr64(data_p, index)			(&((data_p)->ptr[index]))
#define dataGetPtr64(data_p)				((data_p)->ptr)
#define dataSetPtr64(data_p, Ptr)			((data_p)->ptr = Ptr)

#define dataResetLength64(data_p)			((data_p)->length     = 0)
#define dataResetFormat64(data_p)			((data_p)->format     = WORD_SIZE_64_BITS)
#define dataResetGetIndex64(data_p) 		((data_p)->getIndex   = 0)
#define dataResetPutIndex64(data_p) 		((data_p)->putIndex   = 0)
#define dataResetRWIndex64(data_p)          ((data_p)->RWIndex    = 0)


#ifdef _DEBUG
void     dataWrite64(DATA_HEADER64 *data_p, uint64_t val, int16_t index);
uint64_t dataRead64 (DATA_HEADER64 *data_p, int16_t index);
void     dataPut64  (DATA_HEADER64 *data_p, uint64_t val);
uint64_t dataGet64  (DATA_HEADER64 *data_p);
void     dataSetLength64(DATA_HEADER64 *data_p, uint16_t val);
void     dataSetGetIndex64(DATA_HEADER64 *data_p, uint16_t val);
void     dataSetPutIndex64(DATA_HEADER64 *data_p, uint16_t val);
void     dataSetRWIndex64(DATA_HEADER64 *data_p, uint16_t val);
#else
#define dataWrite64(data_p, val, index)		((data_p)->ptr[index] = val)
#define dataRead64(data_p,index)			((data_p)->ptr[index])
#define dataPut64(data_p,val)				((data_p)->ptr[(data_p)->putIndex++] = val)
#define dataGet64(data_p)					((data_p)->ptr[(data_p)->getIndex++])
#define dataSetLength64(data_p, val)		((data_p)->length     = val)
#define dataSetGetIndex64(data_p, val)		((data_p)->getIndex   = val)
#define dataSetPutIndex64(data_p, val)		((data_p)->putIndex   = val)
#define dataSetRWIndex64(data_p, val)       ((data_p)->RWIndex    = val)
#endif

//this section has been added for simplicity of use.
//if we want to change the standarad data format, its sufficient
//to do it here.
//Note: all of those defines should stay in sync with the
//      DATA_HEADER defined in MacLink.h i.e. if we
//      change the DATA_HEADER to be 32bits instead of 08bits
//      the defines below should be changed to 32bits as well.
#define dataSetFormat_                      dataSetFormat08
#define dataSetLength_                      dataSetLength08
#define dataSetMaxLength_                   dataSetMaxLength08
#define dataSetGetIndex_                    dataSetGetIndex08
#define dataSetPutIndex_                    dataSetPutIndex08
#define dataSetRWIndex_                     dataSetRWIndex08

#define dataFormat_                         dataFormat08
#define dataLength_                         dataLength08
#define dataMaxLength_                      dataMaxLength08
#define dataGetIndex_                       dataGetIndex08
#define dataPutIndex_                       dataPutIndex08
#define dataRWIndex_                        dataRWIndex08

#define dataPtr_							dataPtr08
#define dataGetPtr_							dataGetPtr08
#define dataSetPtr_							dataSetPtr08

#define dataResetLength_                    dataResetLength08
#define dataResetFormat_                    dataResetFormat08
#define dataResetGetIndex_                  dataResetGetIndex08
#define dataResetPutIndex_                  dataResetPutIndex08
#define dataResetRWIndex_                   dataResetRWIndex08

#define dataAlign_                          dataAlign08         //function
#define dataRealloc_                        dataRealloc08       //function
#define dataAdd_                            dataAdd08           //function
#define dataRemove_                         dataRemove08        //function
#define dataInitToDefault_                  dataInitToDefault08 //function
#define dataWrite_                          dataWrite08
#define dataRead_                           dataRead08
#define dataPut_                            dataPut08
#define dataGet_                            dataGet08

//-------------------------------------------------------------------------
// structures
//-------------------------------------------------------------------------
#ifdef __TI_COMPILER_VERSION__

#ifdef DEBUG_ALLOC_FREE
typedef struct MemoryAllocationHistory_t {
//	uint16_t callingFunctionRecorded;
//	uint16_t callingFunction;
	uint8_t  *pData;
	uint16_t allocationSize;
} MEMORY_ALLOCATION_HISTORY;
#endif

#endif

//-------------------------------------------------------------------------
// Global variables redeclaration
//-------------------------------------------------------------------------
#ifdef __TI_COMPILER_VERSION__

#ifdef DEBUG_ALLOC_FREE
extern uint8_t  gMemHistoryIndex;
extern MEMORY_ALLOCATION_HISTORY gMemAllocationHistory[];
#endif

#endif

//-------------------------------------------------------------------------
// Function define
//-------------------------------------------------------------------------
uint16_t dataAlign01  (DATA_HEADER01 *data_p, uint16_t byteBoundary);
uint16_t dataRealloc01(DATA_HEADER01 *data_p, uint16_t maxLength);
uint16_t dataAdd01	  (DATA_HEADER01 *data_p, uint16_t maxLength);
uint16_t dataRemove01 (DATA_HEADER01 *data_p);
void     dataInitToDefault01(DATA_HEADER01 *data_p);
uint32_t dataGet01	  (DATA_HEADER01 *data_p, uint16_t nBitLength);
uint16_t dataPut01	  (DATA_HEADER01 *data_p, uint32_t word, uint16_t nBitLength);
uint32_t dataRead01   (DATA_HEADER01 *data_p, uint16_t nBitLength, uint16_t nBitPos);
uint16_t dataWrite01  (DATA_HEADER01 *data_p, uint32_t word, uint16_t nBitLength, uint16_t nBitPos);
uint16_t dataCopy01   (DATA_HEADER01 *dataSrc_p, uint16_t dataSrcIndex,	DATA_HEADER01 *dataDst_p, uint16_t dataDstIndex, uint16_t length);
uint16_t dataShift01  (DATA_HEADER01 *data_p);

uint16_t dataAlign08  (DATA_HEADER08 *data_p, uint16_t byteBoundary);
uint16_t dataRealloc08(DATA_HEADER08 *data_p, uint16_t maxLength);
uint16_t dataAdd08    (DATA_HEADER08 *data_p, uint16_t dataLength);
uint16_t dataRemove08 (DATA_HEADER08 *data_p);
uint16_t dataCopy08   (DATA_HEADER08 *dataSrc_p, uint16_t dataSrcIndex, DATA_HEADER08 *dataDst_p,uint16_t dataDstIndex,uint16_t length);
uint16_t dataShift08  (DATA_HEADER08 *data_p);
void     dataInitToDefault08(DATA_HEADER08 *data_p);

uint16_t dataAlign16  (DATA_HEADER16 *data_p, uint16_t byteBoundary);
uint16_t dataRealloc16(DATA_HEADER16 *data_p, uint16_t maxLength);
uint16_t dataAdd16    (DATA_HEADER16 *data_p, uint16_t dataLength);
uint16_t dataRemove16 (DATA_HEADER16 *data_p);
uint16_t dataCopy16   (DATA_HEADER16 *dataSrc_p, uint16_t dataSrcIndex, DATA_HEADER16 *dataDst_p,uint16_t dataDstIndex,uint16_t length);
uint16_t dataShift16  (DATA_HEADER16 *data_p);
void     dataInitToDefault16(DATA_HEADER16 *data_p);

uint16_t dataAlign32  (DATA_HEADER32 *data_p, uint16_t byteBoundary);
uint16_t dataRealloc32(DATA_HEADER32 *data_p, uint16_t maxLength);
uint16_t dataAdd32    (DATA_HEADER32 *data_p, uint16_t dataLength);
uint16_t dataRemove32 (DATA_HEADER32 *data_p);
uint16_t dataCopy32   (DATA_HEADER32 *dataSrc_p, uint16_t dataSrcIndex, DATA_HEADER32 *dataDst_p,uint16_t dataDstIndex,uint16_t length);
uint16_t dataShift32  (DATA_HEADER32 *data_p);
void     dataInitToDefault32(DATA_HEADER32 *data_p);

uint16_t dataAlign64  (DATA_HEADER64 *data_p, uint16_t byteBoundary);
uint16_t dataRealloc64(DATA_HEADER64 *data_p, uint16_t maxLength);
uint16_t dataAdd64    (DATA_HEADER64 *data_p, uint16_t dataLength);
uint16_t dataRemove64 (DATA_HEADER64 *data_p);
uint16_t dataCopy64   (DATA_HEADER64 *dataSrc_p, uint16_t dataSrcIndex, DATA_HEADER64 *dataDst_p,uint16_t dataDstIndex,uint16_t length);
uint16_t dataShift64  (DATA_HEADER64 *data_p);
void     dataInitToDefault64(DATA_HEADER64 *data_p);

#endif //  DATAAPI_H
