/****************************************************************************
*    Proprietary Data of Adapitve Networks Inc.
*              All rights reserved
*
* Notice:
*		This file contains trade secret data which
*		is the property of Adpative Networks Inc.  This
*		file may not be used for any purpose unless
*		permission is granted in writing by an officier
*		of Adapitve Networks Inc.
*
*
* Filename
*		interfaces.h
*
* Description:
*		descirption of the interfaces that are attached to the
*		cortex and the C6000
*
* NOTE       :
*
* Revision History:
*		Khaled Saab 	6/13/2009	Creation of the file
****************************************************************************/
#ifndef _INTERFACES_H_
#define _INTERFACES_H_
//include files
#include "../Shared/targets.h"

//see corresponding hot bit coding in DSP1000.h
#define _EP_CORES 0
#define ENDPOINT_DSP_CORE_0				(_EP_CORES+0)	//0	same as ICC_CORE_0
#define ENDPOINT_DSP_CORE_1				(_EP_CORES+1)	//1	same as ICC_CORE_1
#define ENDPOINT_DSP_CORE_2				(_EP_CORES+2)	//2	same as ICC_CORE_2
#define ENDPOINT_DSP_CORE_3				(_EP_CORES+3)	//3	same as ICC_CORE_3
#define ENDPOINT_DSP_CORE_4				(_EP_CORES+4)	//4	same as ICC_CORE_4
#define ENDPOINT_DSP_CORE_5				(_EP_CORES+5)	//5	same as ICC_CORE_5
#define ENDPOINT_CORTEX_CORE			(_EP_CORES+6)	//6
#define ENDPOINT_DSP_CORES				(_EP_CORES+7) 	//7
#define ENDPOINT_ALL_CORES				(_EP_CORES+8)	//8

// ENDPOINT replaces 'interface' as the preferred way of referring
// to these destinations or targets since an interface could be
// ethernet, or spi, but more than one virtual or actual device or
// channel could exist per physical interface, we use the term
// endpoint to refer to the final device destination (unique endpoint
// value per destination below)
#define _EP_HOST 10
#define ENDPOINT_POWERLINE				_EP_HOST+0 //10 = 0x0A Powerline
#define ENDPOINT_UART0                  _EP_HOST+1 //11 = 0x0B RS232
#define ENDPOINT_UART1                  _EP_HOST+2 //12 = 0x0C RS232
#define ENDPOINT_UART2                  _EP_HOST+3 //13 = 0x0D RS232
#define ENDPOINT_UART3                  _EP_HOST+4 //14 = 0x0E RS485
#define ENDPOINT_ETHERNET				_EP_HOST+5 //15 = 0x0F
#define ENDPOINT_USB                    _EP_HOST+6 //16 = 0x10

// ---- EPI/HPI ENDPOINTS
#define _EP_EPI 20
#define ENDPOINT_EPI_HPI                _EP_EPI+0  //20 = 0x14 EPI to HPI
#define ENDPOINT_EPI_ADC				_EP_EPI+1  //21 = 0x15 EPI to ADC
#define ENDPOINT_HPI_EPI                _EP_EPI+2  //22 = 0x16 HPI to EPI

// ---- SPI ENDPOINTS:
#define _EP_SPI 25
#define ENDPOINT_SPI_DAC				_EP_SPI+0  //25 = 0x19
#define ENDPOINT_SPI_FLASH1			 	_EP_SPI+1  //26 = 0x1A
#define ENDPOINT_SPI_FLASH2		  	 	_EP_SPI+2  //27 = 0x1B

// ---- I2C ENDPOINTS:
#define _EP_I2C 30

#define ENDPOINT_I2C_DSP 				_EP_I2C+0  //30 = 0x1E
#define ENDPOINT_I2C_CORTEX				_EP_I2C+1  //31 = 0x1F

#define ENDPOINT_VOLTAGE_5_0		 	_EP_I2C+2  //32 = 0x20
#define ENDPOINT_VOLTAGE_1_8		 	_EP_I2C+3  //33 = 0x21
#define ENDPOINT_VOLTAGE_3_3	 		_EP_I2C+4  //34 = 0x22
#define ENDPOINT_VOLTAGE_1_1	 		_EP_I2C+5  //35 = 0x23

#define ENDPOINT_I2C_BUS_EXPANDER_1		_EP_I2C+8  //38 = 0x26	AP44 AP45 AP46
#define ENDPOINT_I2C_BUS_EXPANDER_2		_EP_I2C+9  //39 = 0x27	          AP46
#define ENDPOINT_I2C_BUS_EXPANDER_3		_EP_I2C+10 //40 = 0x28	     AP45

#define ENDPOINT_HUMIDIY				_EP_I2C+11 //41 = 0x29
#define ENDPOINT_TEMPERATURE			_EP_I2C+12 //42 = 0x2A
#define ENDPOINT_ACCELEROMETER			_EP_I2C+13 //43 = 0x2B

#define ENDPOINT_EEPROM  		     	_EP_I2C+14 //44 = 0x2C


//Maximal valid number of end points
#define MAX_VALID_ENDPOINT 				45
#define ENDPOINT_REPLY					0xFF		//generic return to sender endpoint


#endif //_INTERFACES_H_




