/****************************************************************
*
*    Proprietary Data of Adapitve Networks Inc.
*              All rights reserved
*
* Notice:
*		This file contains trade secret data which
*		is the property of Adpative Networks Inc.  This
*		file may not be used for any purpose unless
*		permission is granted in writing by an officier
*		of Adapitve Networks Inc.
*
*
* Filename
*		packetAPI.h
*
* Description:
*		packet header file ...
*
* Revision History:
*		Khaled Saab		Oct 19th 2010   original version
****************************************************************/
#ifndef __PACKETAPI_H
#define __PACKETAPI_H

//-------------------------------------------------------------------------
// Include header Files
//-------------------------------------------------------------------------
#include <stdint.h>		//definitions like uint32_t, ...

#ifdef CORTEX_APP
#include <rtl.h>		//mutex
#endif

#include "../Shared/MacLink.h"
#include "../Shared/dataAPI.h"

//-------------------------------------------------------------------------
// Global defines
//-------------------------------------------------------------------------
// size of gPacketHeader array
//filter core, sync core, edec core, and HPI core don't need a large number
//of packets.
//DSP queues
#ifdef __TI_COMPILER_VERSION__
#define PACKET_MAX_NUM				64		//maximal number of packets (all interfaces)
#else
#define PACKET_MAX_NUM				32		//maximal number of packets (all interfaces)
#endif										//Note: the larger the number of packet
											//		the slower the system is: it takes
											//		more cycles for queueApi and
											//		packetValid function to run
//packet timeOut in msec
#define PACKET_DEFAULT_TTL			10000  	// 10 sec
#define PACKET_MIN_TTL				  100	//100 msec
#define PACKET_INF_TTL				0xFFFFFFFF	//do not expire (32bits)

#define FRAME_TYPE_MASK				0x07	//make sure that they corrolate with
#define FRAME_SUBTYPE_MASK			0x1F	//HEADER_MAC_TYPE_SIZE and
											//HEADER_MAC_SUBTYPE_SIZE in maclink.h

//define the status of the packet (hot bit coded)
//Note: PACKET_READY_STATE and PACKET_DONE_STATE bits should not be both set
#define PACKET_DEFAULT_STATE		0x0000	//default value
#define PACKET_IN_USE_STATE			0x0001	//packet is in use
#define PACKET_READY_STATE 			0x0002	//packet is ready for processing
#define PACKET_ACTIVE_STATE			0x0004	//packet is being processed
#define PACKET_DONE_STATE			0x0008	//packet has been processed
											//this state is used to keep
											//the packet in the buffer
											//(History)
#define PACKET_TTL_EXPIRED_STATE	0x0100	//packet TTL has expired
#define PACKET_LOCK_STATE			0x0200	//packet is locked
											//used by upper layer to tag
											//a packet
// packet status access
#define packetQueue(ptr)				((ptr)->status.queue)
#define packetNext(ptr)                 ((ptr)->status.next)
#define packetState(ptr)                ((ptr)->status.state)
#define packetNumBlocks(ptr)			((ptr)->status.numBlocks)
#define packetBlockTx(ptr)				((ptr)->status.blockRxTx)
#define packetBlockRx(ptr)				((ptr)->status.blockRxTx)
#define packetBlockTxTag(ptr)			((ptr)->status.blockRxTxTag)

//status.blockRxTx[number of block] is a dual purpose register binary
//register [TRUE, FALSE]:
//	- the transmitter uses it to keep track of the blocks that has
//	  been transmitted AND acked.
//	- the receiver uses it to keep track of the blocks received (packet
//	  reassembly).
#define packetBlockTxGet(          ptr, index) ((ptr)->status.blockRxTx &   ((uint64_t)1 << index) ? TRUE : FALSE)
#define packetBlockTxSet(          ptr, index) ((ptr)->status.blockRxTx |=  ((uint64_t)1 << index))
#define packetBlockTxClear(        ptr, index) ((ptr)->status.blockRxTx &= ~((uint64_t)1 << index))
#define packetBlockTxClearAll(     ptr       ) ((ptr)->status.blockRxTx  =  ((uint64_t)0         ))
#define packetBlockRxGet(          ptr, index) ((ptr)->status.blockRxTx &   ((uint64_t)1 << index) ? TRUE : FALSE)
#define packetBlockRxGetAll(	   ptr		 ) ((ptr)->status.blockRxTx)
#define packetBlockRxSet(          ptr, index) ((ptr)->status.blockRxTx |=  ((uint64_t)1 << index))
#define packetBlockRxClear(        ptr, index) ((ptr)->status.blockRxTx &= ~((uint64_t)1 << index))
#define packetBlockRxClearAll(     ptr       ) ((ptr)->status.blockRxTx  =  ((uint64_t)0         ))

//status.blockRxTxCount[number of blocks] is a dual prupose register:
//	- the transmitter uses it to keep track of the blocks that has been
//	  transmitted AND NOT acked yet.
//	- the receiver uses it to keep count of the block received (stat
//	  purposes only for now)
#define packetBlockTxTagGet(       ptr, index) ((ptr)->status.blockRxTxTag &   ((uint64_t)1 << index) ? TRUE : FALSE)
#define packetBlockTxTagSet(       ptr, index) ((ptr)->status.blockRxTxTag |=  ((uint64_t)1 << index))
#define packetBlockTxTagClear(     ptr, index) ((ptr)->status.blockRxTxTag &= ~((uint64_t)1 << index))
#define packetBlockTxTagClearAll(  ptr       ) ((ptr)->status.blockRxTxTag  =  ((uint64_t)0         ))
#define packetBlockRxTagGet(       ptr, index) ((ptr)->status.blockRxTxTag &   ((uint64_t)1 << index) ? TRUE : FALSE)
#define packetBlockRxTagSet(       ptr, index) ((ptr)->status.blockRxTxTag |=  ((uint64_t)1 << index))
#define packetBlockRxTagClear(     ptr, index) ((ptr)->status.blockRxTxTag &= ~((uint64_t)1 << index))
#define packetBlockRxTagClearAll(  ptr       ) ((ptr)->status.blockRxTxTag  =  ((uint64_t)0         ))

//packet phy access
#define packetMacLength(ptr, index)		((ptr)->phy.macLength)
#define packetChLoading(ptr) 			((ptr)->phy.chLoading)
#define packetgain(			ptr)		((ptr)->phy.gain)

//packet mac access
#define packetMacPtr(		ptr)		(&((ptr)->mac))
#define packetPayloadLength(ptr)		((ptr)->mac.payloadLength)
#define packetVersion(		ptr)		((ptr)->mac.version)
#define packetType(			ptr)		((ptr)->mac.type)
#define packetSubType(		ptr)		((ptr)->mac.subtype)
#define packetSeqNum(		ptr)		((ptr)->mac.packetSeqNum)
#define packetPriority(		ptr)		((ptr)->mac.priority)
#define packetFlags(		ptr)		((ptr)->mac.flags)
#define packetDstAddr(		ptr)		((ptr)->mac.dstAddr)
#define packetSrcAddr(		ptr)		((ptr)->mac.srcAddr)

#define packetTotalLength(	ptr)		((ptr)->mac.packetLength)
#define packetTTL(			ptr)		((ptr)->mac.msToLive)
#define packetCrc(			ptr)		((ptr)->mac.packetCrc)
#define packetThreshold(	ptr)		((ptr)->mac.threshold)
#define packetRoutingAddr(	ptr)		((ptr)->mac.routingAddr)
#define packetFrameAck(		ptr)		((ptr)->mac.frameAck)
#define packetPacketAck(	ptr)		((ptr)->mac.packetAck)

#define packetClearTTL(		ptr)		{(ptr)->mac.msToLive = 0; (ptr)->status.state |= PACKET_TTL_EXPIRED_STATE;}

//packet data access
#define packetDataPtr(		ptr)		(&((ptr)->data))
#define packetDataRemove(	ptr)		dataRemove_(packetDataPtr(ptr))

#define packetSetPtr(		Ptr, addr)	((Ptr)->data.ptr = (uint8_t *) addr)
#define packetGetPtr(		Ptr)		((Ptr)->data.ptr)
#define packetSetPtrSave(	Ptr, addr)	((Ptr)->data.ptrSave = (uint8_t *) addr)
#define packetGetPtrSave(	Ptr)		((Ptr)->data.ptrSave)
#define packetOffsetPtr(	Ptr, offset)((Ptr)->data.ptr += offset)
#define packetPtr(			Ptr, index)	(&((Ptr)->data.ptr[index]))

#define packetPutIndex(		ptr)		((ptr)->data.putIndex)
#define packetGetIndex(		ptr)		((ptr)->data.getIndex)
#define packetSetGetIndex(	ptr, index)	((ptr)->data.getIndex = index)
#define packetSetPutIndex(	ptr, index)	((ptr)->data.putIndex = index)
#define packetResetGetIndex(ptr)		((ptr)->data.getIndex = 0)
#define packetResetPutIndex(ptr)		((ptr)->data.putIndex = 0)
#define packetRWIndex(		ptr)		((ptr)->data.RWIndex)
#define packetResetRWIndex(	ptr)		((ptr)->data.RWIndex  = 0)
#define packetFormat(		ptr)		((ptr)->data.format)

#define packetLength(		ptr)		((ptr)->data.length)
#define packetMaxLength(	ptr)		((ptr)->data.maxLength)
#define packetSetLength(	ptr, len)	((ptr)->data.length = len)
#define packetGetLength(	ptr)		((ptr)->data.length)
#define packetSetMaxLength(	ptr, len)	((ptr)->data.maxLength = len)
#define packetGetMaxLength(	ptr)		((ptr)->data.maxLength)

//packet interface access
#define packetDstEP(		ptr)		((ptr)->endPoint.dstEndPoint)
#define packetSrcEP(		ptr)		((ptr)->endPoint.srcEndPoint)
#define packetAddrEP(		ptr)		((ptr)->endPoint.addrEndPoint)
#define packetCmd(			ptr)		((ptr)->endPoint.command)

//-------------------------------------------------------------------------
// Global varialbes redeclaration
//-------------------------------------------------------------------------
extern PACKET_HEADER gPacketHeader[PACKET_MAX_NUM];
extern uint16_t      gPacketAvailable;
extern uint16_t      gPacketIndex;

//-------------------------------------------------------------------------
// Function define
//-------------------------------------------------------------------------
uint32_t 	packetGet      	(PACKET_HEADER *packet_p);
uint16_t 	packetPut      	(PACKET_HEADER *packet_p, uint32_t word);
uint32_t 	packetRead     	(PACKET_HEADER *packet_p, int16_t index);                  //XXX use #define
uint16_t 	packetWrite    	(PACKET_HEADER *packet_p, uint32_t word, int16_t index);   //XXX use #define

uint16_t 	packetAvailableCount(void);
uint16_t 	packetAlign    	(PACKET_HEADER *packet_p, uint16_t byteBoundary);
uint16_t 	packetRealloc  	(PACKET_HEADER *packet_p, uint16_t dataLength);
PACKET_HEADER *packetAcquire(uint16_t dataLength);

//User should be carefull using this funciton.
//This function is NOT part of the standard packet API and thus its NOT
//managed by it.
//It was added to give the user the capability to create his own packet.
uint16_t 	packetAdd		(PACKET_HEADER *packet_p, uint16_t maxLength);
uint16_t 	packetCopy		(PACKET_HEADER **packetDst_pp, PACKET_HEADER *packetSrc_p);
uint16_t 	packetRemove   	(PACKET_HEADER *packet_p);

void 	 	packetInit     	(void);
void 	 	packetInitToDefault(PACKET_HEADER *packet_p);

uint16_t	packetValid		(PACKET_HEADER *packet_p);

#endif //  __PACKETAPI_H
