/****************************************************************
 *
 *    Proprietary Data of Adaptive Networks Inc.
 *              All rights reserved
 *
 * Notice:
 *		This file contains trade secret data which
 *		is the property of Adaptive Networks Inc.  This
 *		file may not be used for any purpose unless
 *		permission is granted in writing by an officer
 *		of Adaptive Networks Inc.
 *
 *
 * Filename
 *		packetRouteCortex.c
 *
 * Description:
 *
 * History:
 *		 	Siyi Tan Feb 14th 2013 Creation
 ****************************************************************/
#include <stdint.h>					//declaration of uint8_t int16_t...

#include "../Shared/DSP1000.h"
#include "../Shared/interfaces.h"
#include "../Shared/queueAPI.h"
#include "../Shared/packetRouteCortex.h"
#include "../cortex/our_drivers/taskMgr.h"
/**************************************************************************
* Function     : packetRoute
* Description  : route a packet to the queue and set the event
*
* Input        : packet_p	: packet to be routed
* Return       : SUCCESS/FAIL
* History      :
* Siyi Tan		Feb 14th 2013 	Created base on EthernetRoutePacket
**************************************************************************/
uint8_t packetRoute(PACKET_HEADER *packet_p){

	uint8_t status;

	if (packet_p == NULL){
		return FAIL;
	}
	//---------------------------------------------
	//put the packet on the proper queue
	//we have 2 conditions:
	//if the packet belongs to a queue
	//	move it from 1 queue to another
	//else
	//	add it to the proper queue
	//---------------------------------------------
	if(queueValid(packetQueue(packet_p))) {
		//if dst is the Ethernet Ep
		//	move the packet to the Ethernet Tx queue
		if(packetDstEP(packet_p) == ENDPOINT_ETHERNET) {
			status = queueMove(packet_p, &gQueueEthernetTx);
			if (status == SUCCESS){
				os_evt_set(TASK_EVENT_TX_FLAG, gTaskEthernetTx_ID);
			}
		}
		//if dst is not valid
		//	return FAIL
		else {
			status = FAIL;
		}
	}
	else {

		//if dst is the Ethernet Ep
		//	move the packet to the Ethernet Tx queue
		if(packetDstEP(packet_p) == ENDPOINT_ETHERNET) {
			status = queueAdd(&gQueueEthernetTx, packet_p);
			if (status == SUCCESS){
				os_evt_set(TASK_EVENT_TX_FLAG, gTaskEthernetTx_ID);
			}
		}
		//if dst is not valid
		//	return FAIL
		else {
			status = FAIL;
		}
	}
	return status;
}
