/****************************************************************
*
*    Proprietary Data of Adapitve Networks Inc.
*              All rights reserved
*
* Notice:
*		This file contains trade secret data which
*		is the property of Adpative Networks Inc.  This
*		file may not be used for any purpose unless
*		permission is granted in writing by an officier
*		of Adapitve Networks Inc.
*
*
* Filename
*		QueueAPI.h
*
* Description:
*		packet header file ...
*
* Revision History:
*		Khaled Saab		Oct 19th 2010   moved from Queue_driver.c
****************************************************************/
#ifndef __QUEUE_API_H
#define __QUEUE_API_H

//-------------------------------------------------------------------------
// Include header Files
//-------------------------------------------------------------------------
#include <stdint.h>  	//definitions like uint32_t, ...

#ifdef CORTEX_APP
#include <rtl.h>		//mutex
#endif

#include "../Shared/MacLink.h"

//-------------------------------------------------------------------------
// Global defines
//-------------------------------------------------------------------------
//define the burst maximal size and burst default size
//this values is in bytes (number of bytes to transmit per burst)
#define     BURST_DEFAULT_SIZE    256
#define     BURST_MAX_SIZE       1024    //maximal number of uDMA transfer

#define ENDPOINT_ETHERNET_TX_ENABLE     	TRUE   //ethernet does not support burst
#define ENDPOINT_ETHERNET_RX_ENABLE     	TRUE
#define ENDPOINT_ETHERNET_TX_MAX_LENGTH 	5
#define ENDPOINT_ETHERNET_RX_MAX_LENGTH 	5
#define ENDPOINT_ETHERNET_BURST_SIZE    	BURST_MAX_SIZE

//----------------------------------------------
//shared queues (both Cortex and C6000)
//define the ENDPOINTs characteristics:
//      - Tx enable
//      - Rx enable
//      - Tx max number of packets allowed
//      - Rx max number of packets allowed
//      - allow or disallow burst mode
//----------------------------------------------
#define ENDPOINT_SNMP_RX_ENABLE   				TRUE
#define ENDPOINT_SNMP_RX_MAX_LENGTH             10
#define ENDPOINT_SNMP_BURST_SIZE                BURST_MAX_SIZE

//-----------------------------------------------------------------------------
//	Data Structures
//-----------------------------------------------------------------------------
#define queueActivePacket(ptr)          ((ptr)->activePacket)
#define queueActivePacketClear(ptr)     ((ptr)->activePacket = NULL)
#define queueNext(ptr)                  ((ptr)->next)
#define queueLength(ptr)                ((ptr)->length)
#define queueMaxLength(ptr)             ((ptr)->maxLength)
#define queueBurstSize(ptr)             ((ptr)->burstSize)

typedef struct queueHeader_t {
	PACKET_HEADER   *activePacket;  //pointer to the packet that is in
                                    //process
    PACKET_HEADER   *next;          //pointer to the next packet in the
                                    //queue (sorted chained list)

    uint8_t         length;         //number of packets in the queue
    uint8_t         maxLength;      //maximal number of packets allowed
    uint16_t        burstSize;      //set to BURST_MAX_SIZE do disable burst

}QUEUE_HEADER;
#define QUEUE_HEADER_DEFAULT {NULL, NULL, 0, 0, 0}

//-------------------------------------------------------------------------
// Global varialbes redeclaration
//-------------------------------------------------------------------------
extern QUEUE_HEADER   gQueueEthernetTx;    //send to Ethernet
extern QUEUE_HEADER   gQueueEthernetRx;    //receive from Ethernet

//-------------------------------------------------------------------------
// Function define
//-------------------------------------------------------------------------
uint16_t queueValid(		QUEUE_HEADER *queueHeader_p);
void queueInit(				void);
void queueInitToDefault(    QUEUE_HEADER  *queueHeader_p);
uint16_t queueSanityCheck(	QUEUE_HEADER *queueHeader_p);
uint16_t queueReset(        QUEUE_HEADER  *queueHeader_p);

void     queueRemoveExpiredTTL(void);
uint16_t queueClearTTL(		QUEUE_HEADER  *queueHeader_p);

PACKET_HEADER *queueGet(    QUEUE_HEADER  *queueHeader_p,
                            uint16_t setFlag,
							uint16_t clearFlag);

uint16_t queueAdd(			QUEUE_HEADER  *queueHeader_p,
                            PACKET_HEADER *packet_p);
uint16_t queueRemove(		PACKET_HEADER *packet_p);
uint16_t queueMove  (       PACKET_HEADER *packet_p,
                            QUEUE_HEADER  *queueHeaderDst_p);
#endif //  __QUEUE_API_H
