/****************************************************************************
*    Proprietary Data of Adapitve Networks Inc.
*              All rights reserved
*
* Notice:
*		This file contains trade secret data which
*		is the property of Adpative Networks Inc.  This
*		file may not be used for any purpose unless
*		permission is granted in writing by an officier
*		of Adapitve Networks Inc.
*
*
* Filename
*		targets.h
*
* Description:
*		target the code compilation to:
*			Enable/Disable the endpoints (Ethernet, RS232, RS485...)
*			Enable/Disable the watchdog (HPI, Heart beat...)
*			Enable/Disable the boot polling loop
*
* NOTE       :
*
* Revision History:
*	Khaled Saab 	April 31th 2012	Creation of the file
****************************************************************************/
#ifndef _TARGETS_H_
#define _TARGETS_H_

//compile flags
//if the variable is defined, disable the corresponding feature.
//for example: if #define DISABLE_UART0_EP, the Cortex UART0 will be disabled
#undef	DISABLE_UART0_EP				//Cortex RS232 snmp
#undef	DISABLE_UART3_EP				//Cortex RS485 modbus
#undef	DISABLE_ETHERNET_EP				//Cortex Ethernet port
#undef 	DISABLE_SPI_DAC_EP				//Cortex writting to SPI DAC
#undef 	DISABLE_SPI_FLASH_EP			//Cortex writting to SPI flash
#undef  DISABLE_EPI_HPI_EP				//Cortex to DSP
#undef  DISABLE_EPI_ADC_EP				//Cortex reading the ADC
#define	DISABLE_USB_EP					//Cortex USB

#undef	DISABLE_I2C_EP					//I2C
 #undef  DISABLE_I2C_BUS_EXPANDER_EP	//I2C bus expander
 #undef  FLIP_I2C_BUS_EXPANDER_POLARITY	//I2C bus expander polarity flipped
 #undef  DISABLE_I2C_VOLTAGE_5_0		//I2C voltage monitor (5.0v)
 #undef  DISABLE_I2C_VOLTAGE_1_8		//I2C voltage monitor (1.8v)
 #undef  DISABLE_I2C_VOLTAGE_3_3		//I2C voltage monitor (3.3v)
 #undef  DISABLE_I2C_VOLTAGE_1_1		//I2C voltage monitor (1.1v)

#undef	DISABLE_TIMER					//Cortex timers

#undef  DISABLE_WDG						//watchdog (Cortex & DSP)
#undef  DISABLE_WDG_HPI_STALL			//HPI stall monitoring
#undef  DISABLE_WDG_ADC_STALL			//ADC stall monitoring
#undef	DISABLE_WDG_HEART_BEAT			//Heart beat monitoring

//modem version select
#define AP44					44
#define AP45					45		//Euro card: same as AP44 but with I2C bus expander
#define AP46					46

//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
//select the client
//configure the modem based on client
//----------------------------------------------------------------------
//----------------------------------------------------------------------
//----------------------------------------------------------------------
#define CLIENT_TEST_MODE		0
#define CLIENT_JOY				1
#define CLIENT_ONE_SUBSEA		2
#define CLIENT_GE				3
#define CLIENT_OCEANEERING		4
#define CLIENT_SOLMEAD			5
#define CLIENT					CLIENT_TEST_MODE

//Joy
//	AP46, 1.3x frequency, and I2C inverted
//----------------------------------------------------------------------
#if(CLIENT == CLIENT_JOY)				//client is Joy
#ifndef  BOARD_VER
 #define BOARD_VER				AP46	//AP46
#endif
#ifndef  FLIP_I2C_BUS_EXPANDER_POLARITY
 #define FLIP_I2C_BUS_EXPANDER_POLARITY	//invert I2C bus expander polarity
#endif									//i.e. default == Slave
#ifndef  BLOCK_NUM_PER_FRAME
 #define BLOCK_NUM_PER_FRAME 	5	//5blocks/frame (BPSK)
#endif

//One Subsea
//	AP46, low frequencies
//----------------------------------------------------------------------
#elif(CLIENT == CLIENT_ONE_SUBSEA)		//client is One Subsea
#ifndef  BOARD_VER
 #define BOARD_VER				AP45	//AP45
#endif
#ifndef  FLIP_I2C_BUS_EXPANDER_POLARITY
 #define FLIP_I2C_BUS_EXPANDER_POLARITY	//invert I2C bus expander polarity
#endif									//i.e. default == Slave
#ifndef  DIRECTED_DISCOVERY_DEFAULT		//power up in directed discovery
 #define DIRECTED_DISCOVERY_DEFAULT 	//mode
#endif
#ifndef  DISABLE_UART3_EP
 #define DISABLE_UART3_EP				//RS485/Modbus not used
#endif

//GE
//	AP44, 1x freq
//----------------------------------------------------------------------
#elif(CLIENT == CLIENT_GE)				//client is GE
#ifndef  BOARD_VER
 #define BOARD_VER				AP44	//AP44
#endif


//Oceaneering
//----------------------------------------------------------------------
#elif(CLIENT == CLIENT_OCEANEERING)		//client is Oceannering
#ifndef  BOARD_VER
 #define BOARD_VER				AP46	//AP46
#endif

#ifndef  BLOCK_PAYLOAD_SIZE
 #define BLOCK_PAYLOAD_SIZE		24		//number of bytes in a block (pre-encoding)
#endif
#ifndef  BLOCK_NUM_PER_FRAME
 #define BLOCK_NUM_PER_FRAME 	5		//3blocks/frame
#endif
#ifndef  BLOCK_MAX_NUM_PER_PACKET
 #define BLOCK_MAX_NUM_PER_PACKET	64	//max number of blocks
#endif
#ifndef  BLOCK_MAX_NUM_PER_FRAME
 #define BLOCK_MAX_NUM_PER_FRAME	64
#endif


//Default client configuration
//----------------------------------------------------------------------
#else
#ifndef BOARD_VER
 #define BOARD_VER				AP45	//AP46
#endif
#ifndef  DISABLE_WDG_HEART_BEAT			//no heart beat
 #define DISABLE_WDG_HEART_BEAT
#endif
#ifndef  DISABLE_UART3_EP
 #define DISABLE_UART3_EP				//RS485/Modbus not used
#endif

#endif	//client selection





//specific board configuration that applies to all customers
//----------------------------------------------------------------------
//define the board hardware version.
//Note: that this code should reside in Version.h. However, since the
//      BOARD is defined in this file, we had to define the sub version
//      here as well.
#if(BOARD_VER == AP44)
 #define BOARD_SUB_VER		0x02
#elif(BOARD_VER == AP45)
 #define BOARD_SUB_VER		0x03
#elif(BOARD_VER == AP46)
 #define BOARD_SUB_VER		0x00
#else
 #error "Board version not supported"
#endif

#endif //_TARGETS_H_




