//*****************************************************************************
//
// system_lm3s.c - System initialization file for CMSIS application on a
//                 Luminary Micro Stellaris microcontroller.
//
// Copyright (c) 2009 Luminary Micro, Inc.  All rights reserved.
// Software License Agreement
//
// Luminary Micro, Inc. (LMI) is supplying this software for use solely and
// exclusively on LMI's microcontroller products.
//
// The software is owned by LMI and/or its suppliers, and is protected under
// applicable copyright laws.  All rights are reserved.  You may not combine
// this software with "viral" open-source software in order to form a larger
// program.  Any use in violation of the foregoing restrictions may subject
// the user to criminal sanctions under applicable laws, as well as to civil
// liability for the breach of the terms and conditions of this license.
//
// THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
// OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
// LMI SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
//
// This is part of revision 30 of the Stellaris CMSIS Package.
//
//*****************************************************************************
#include <stdbool.h>
#include <stdint.h>                 //declaration of uint8_t int8_t...
#include <inc/hw_memmap.h>			//base addresses
#include <inc/hw_types.h>
#include <inc/hw_ints.h>
#include <inc/hw_sysctl.h>      	//sys control registers
#include <inc/hw_gpio.h>
#include <driverlib/sysctl.h>
#include <driverlib/interrupt.h>	//interrupts

#include "../../Shared/interfaces.h"
#include "../Interfaces/A_GPIO.h"
#include "../main/system_TM4C129.h"

//-----------------------------------------------------------------------
// global variables
//-----------------------------------------------------------------------
uint32_t gSysCtlClock;  //the system clock

//-------------------------------------------------------------------------
// Function Prototypes
//-------------------------------------------------------------------------
//function declared in sysclt.c
uint32_t MY_SysCtlClockFreqSet(uint32_t ui32Config, uint32_t ui32SysClock);

/**************************************************************************
* Function     : clockInit()
*
* Description  : set the system clock.
*                Currently its set to 100MHz!
* Note         : TI/driverlib/sysctl.c has been updated to generate a
*                400MHz PLL frequency and 100MHz gSysCtlClock.
*                see cortex\StellarisWare\driverlib\sysctl.c
*                Similarlty sysctl.c/SysCtlFrequencyGet( ) has been updated
*                to correct for a rounding error when 100MHz is generated.
* Input        :
* Output  	   :
* Return       :
**************************************************************************/
//add an extra define for 400MHz PLL frequency.
//typically we should add it to the sysctl.h file, but decided to put it
//here and keep TI's file unchanged.
#ifndef SYSCTL_CFG_VCO_400
#define SYSCTL_CFG_VCO_400 0xF2000000
#endif

void clockInit() {

    // Make sure the main oscillator is enabled because this is required by
    // the PHY.  The system must have a 25MHz crystal attached to the OSC
    // pins.  The SYSCTL_MOSC_HIGHFREQ parameter is used when the crystal
    // frequency is 10MHz or higher.
	SysCtlMOSCConfigSet(SYSCTL_MOSC_HIGHFREQ);

	//configure the clock
	//sysctl.c has been updated to generate a 100MHz gSysCtlClock
	//see cortex\StellarisWare\driverlib\sysctl.c
	gSysCtlClock = MY_SysCtlClockFreqSet(
						SYSCTL_USE_PLL    |
						SYSCTL_OSC_MAIN   |
						SYSCTL_XTAL_25MHZ |
						SYSCTL_CFG_VCO_400,
						100000000);		//desired system frequency
}

/**************************************************************************
* Function     : SystemInit()
*
* Description  : Setup the microcontroller system.
*                Initialize the System and update the SystemFrequency variable.
* Input        :
* Output  	   :
* Return       :
**************************************************************************/
void SystemInit (void) {

	clockInit();	// set the clock
    return;
}
