/*
 * Copyright (c) 2005, Swedish Institute of Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the uIP TCP/IP stack
 *
 */
 /****************************************************************
 *
 *    Proprietary Data of Adaptive Networks Inc.
 *              All rights reserved
 *
 * Notice:
 *		This file contains trade secret data which
 *		is the property of Adaptive Networks Inc.  This
 *		file may not be used for any purpose unless
 *		permission is granted in writing by an officer
 *		of Adaptive Networks Inc.
 *
 *
 * Filename
 *		Ethernet_DHCP.h
 *
 * Description: dhcp function
 *
 *
 * History:
 *		Siyi Tan	Jan 14th 2013   original version
 ****************************************************************/
#ifndef __ETHERNET_DHCP_H__
#define __ETHERNET_DHCP_H__
//-------------------------------------------------------------------------
// Include header Files
//-------------------------------------------------------------------------
#include <stdint.h>
#include "../../Shared/packetAPI.h"
//-------------------------------------------------------------------------
// Global defines
//-------------------------------------------------------------------------
//#define UIP_CONF_DHCP_LIGHT

#ifndef DHCPC_SERVER_PORT
#define DHCPC_SERVER_PORT   67
#endif
#ifndef DHCPC_CLIENT_PORT
#define DHCPC_CLIENT_PORT   68
#endif

#define DHCPDISCOVER         1
#define DHCPOFFER            2
#define DHCPREQUEST          3
#define DHCPDECLINE          4
#define DHCPACK              5
#define DHCPNAK              6
#define DHCPRELEASE          7


#define DHCP_STATE_INIT     0x0000000
#define DHCP_STATE_REBOOT   0x0000010
#define DHCP_STATE_SELECT   0x0000100
#define DHCP_STATE_REQUEST  0x0001000
#define DHCP_STATE_BOUND    0x0010000
#define DHCP_STATE_RENEW    0x0100000
#define DHCP_STATE_REBIND   0x1000000

#define BOOTP_BROADCAST     0x8000
#define BOOTP_UNICAST       0x0000

#define DHCP_REQUEST              1
#define DHCP_REPLY                2
#define DHCP_HTYPE_ETHERNET       1
#define DHCP_HLEN_ETHERNET        6
#define DHCP_MSG_LEN            236

#define DHCP_OPTION_SUBNET_MASK   1
#define DHCP_OPTION_ROUTER        3
#define DHCP_OPTION_DNS_SERVER    6
#define DHCP_OPTION_REQ_IPADDR   50
#define DHCP_OPTION_LEASE_TIME   51
#define DHCP_OPTION_MSG_TYPE     53
#define DHCP_OPTION_SERVER_ID    54
#define DHCP_OPTION_REQ_LIST     55
#define DHCP_OPTION_END         255

#define MAX_RESPONSE_TIMEOUT    64000    //64 sec
//-----------------------------------------------------------------------------
//	Data Structures
//-----------------------------------------------------------------------------
struct dhcpc_state {

  uint8_t serverid[4];

  uint16_t lease_time[2];
  uint16_t ipaddr[2];
  uint16_t netmask[2];
  uint16_t dnsaddr[2];
  uint16_t default_router[2];
};

typedef struct dhcp_msg_t {
  uint8_t op;				//operation code
  uint8_t htype;			//hardware type
  uint8_t hlen;				//hardware address length
  uint8_t hops;				//Hops
  uint8_t xid[4];			//transaction ID
  uint16_t secs;			//seconds
  uint16_t flags;			//flags
  uint8_t ciaddr[4];		//client Ip address
  uint8_t yiaddr[4];		//your Ip address
  uint8_t siaddr[4];		//server Ip address
  uint8_t giaddr[4];		//Geateway Ip address
  uint8_t chaddr[16];		//client hardware address
#ifndef UIP_CONF_DHCP_LIGHT
  uint8_t sname[64];		//server name
  uint8_t file[128];		//boot filename
#endif
  uint8_t options[312];		//options
}DHCP_MSG;

//-------------------------------------------------------------------------
// Global variables
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------
// Function define
//-------------------------------------------------------------------------
uint32_t 	DHCPPacketProcess(	PACKET_HEADER *packet_p, uint8_t *EthernetIpAddr_p);
void 		DHCP_discover(		DHCP_MSG *m);
void     	DHCPRequest(		DHCP_MSG *m);
uint32_t 	DHCPConfigure(		uint8_t *EthernetIpAddr_p);
uint8_t 	parse_options(		uint8_t *optptr, int len);
uint8_t 	DHCPParseMsg(		PACKET_HEADER *packet_p);

typedef struct dhcpc_state uip_udp_appstate_t;
#define UIP_UDP_APPCALL dhcpc_appcall
#endif /* __DHCPC_H__ */
