/****************************************************************
 *
 *    Proprietary Data of Adaptive Networks Inc.
 *              All rights reserved
 *
 * Notice:
 *		This file contains trade secret data which
 *		is the property of Adaptive Networks Inc.  This
 *		file may not be used for any purpose unless
 *		permission is granted in writing by an officer
 *		of Adaptive Networks Inc.
 *
 *
 * Filename
 *		Ethernet_Driver.c
 *
 * Description:
 *
 * History:
 *		John Houle	Jan 19th 2010   original version
 *      Khaled Saab Nov 18th 2010   complete re-write. Moved code
 *                                  from other files to here.
 *                                  made it compatible with uDMA
 *                                  read and write.
 *                                  removed all code related to
 *                                  http, tcp... Should be restored.
 ****************************************************************/
#include <rtl.h>			//ntohs ntohl htons htonl
#include <stdbool.h>		//bool
#include <stdint.h>			//declaration of uint8_t int16_t...
#include <string.h>			//definitions memcpy
#include <stdlib.h>

#include <inc/hw_memmap.h>
#include <inc/hw_types.h>
#include <inc/hw_ints.h>
#include <inc/hw_emac.h>
#include <driverlib/udma.h>
#include <driverlib/emac.h>
#include <driverlib/flash.h>
#include <driverlib/interrupt.h>
#include <driverlib/sysctl.h>

#include "../StellarisWare/uip-1.0/uip.h"
#include "../StellarisWare/lwip-1.3.2/src/include/ipv4/lwip/inet_chksum.h"

#include "../../Shared/DSP1000.h"
#include "../../Shared/interfaces.h"
#include "../../Shared/dataAPI.h"
#include "../../Shared/packetAPI.h"
#include "../../Shared/packetRouteCortex.h"
#include "../../Shared/queueAPI.h"

#include "../main/main.h"
#include "../main/system_TM4C129.h"
#include "../Interfaces/A_UDMA.h"
#include "../our_drivers/Ethernet_Driver.h"
#include "../our_drivers/Ethernet_DHCP.h"
#include "../our_drivers/taskMgr.h"

//-----------------------------------------------------------------
//Global variables
//-----------------------------------------------------------------
uint8_t gEthernetMacAddr[6];
uint8_t gEthernetIpAddr[4];
uint8_t gEthernetGateway[4];
uint8_t gEthernetMask[4];


// Ethernet DMA descriptors.
// Although uIP uses a single buffer, the MAC hardware needs a minimum of
// 3 receive descriptors to operate.
tEMACDMADescriptor g_psRxDescriptor[NUM_RX_DESCRIPTORS];
tEMACDMADescriptor g_psTxDescriptor[NUM_TX_DESCRIPTORS];

uint32_t g_ui32RxDescIndex;
uint32_t g_ui32TxDescIndex;

// Transmit and receive buffers.
uint8_t g_pui8RxBuffer[RX_BUFFER_SIZE];
uint8_t g_pui8TxBuffer[TX_BUFFER_SIZE];


/**************************************************************************
* Function     : Ethernet_AddressInit()
*
* Description  : Initialize the Ethenet address
* Input        :
* Output  	   :
* Revision History:
**************************************************************************/
void Ethernet_AddressInit(void) {

    uint32_t ui32User0, ui32User1;

    //-------------------------------------------------------------
    // Get the MAC address from the flash user registers.  If it has not been
    // programmed, then use the default MAC address.
	//-------------------------------------------------------------
    FlashUserGet(&ui32User0, &ui32User1);
    if((ui32User0 == 0xffffffff) || (ui32User1 == 0xffffffff)) {
        // MAC address has not been programmed, use default.
		gEthernetMacAddr[0] = MAC_ADDRESS1;
		gEthernetMacAddr[1] = MAC_ADDRESS2;
		gEthernetMacAddr[2] = MAC_ADDRESS3;
		gEthernetMacAddr[3] = MAC_ADDRESS4;
		gEthernetMacAddr[4] = MAC_ADDRESS5;
		gEthernetMacAddr[5] = MAC_ADDRESS6;
	}
    else {
		gEthernetMacAddr[0] = (ui32User0      ) & 0xff;
		gEthernetMacAddr[1] = (ui32User0 >>  8) & 0xff;
		gEthernetMacAddr[2] = (ui32User0 >> 16) & 0xff;
		gEthernetMacAddr[3] = (ui32User1      ) & 0xff;
		gEthernetMacAddr[4] = (ui32User1 >>  8) & 0xff;
		gEthernetMacAddr[5] = (ui32User1 >> 16) & 0xff;
	}

	//-------------------------------------------------------------
	//load default Ip address as well as the default mask
	//and gateway
	//-------------------------------------------------------------
	gEthernetIpAddr[0]  = DEFAULT_IPADDR0;
	gEthernetIpAddr[1]  = DEFAULT_IPADDR1;
	gEthernetIpAddr[2]  = DEFAULT_IPADDR2;
	gEthernetIpAddr[3]  = DEFAULT_IPADDR3;

	gEthernetGateway[0] = DEFAULT_GATEWAY0;
	gEthernetGateway[1] = DEFAULT_GATEWAY1;
	gEthernetGateway[2] = DEFAULT_GATEWAY2;
	gEthernetGateway[3] = DEFAULT_GATEWAY3;

	gEthernetMask[0]    = DEFAULT_NETMASK0;
	gEthernetMask[1]    = DEFAULT_NETMASK1;
	gEthernetMask[2]    = DEFAULT_NETMASK2;
	gEthernetMask[3]    = DEFAULT_NETMASK3;

}
 /**************************************************************************
* Function     : Ethernet_DescriptorsInit()
*
* Description  : Initialize the transmit and receive DMA descriptors.
*				 We apparently need a minimum of 3 descriptors in each chain.
*				 This is overkill since uIP uses a single, common transmit
*				 and receive buffer so we tag each descriptor with the same
*				 buffer and will make sure we only hand the DMA one descriptor
*				 at a time.
* Input        :
* Output  	   :
* Revision History:
*	Khaled Saab	Aug 11th 2014	ported from TI example code (enet_uip.c)
**************************************************************************/
void Ethernet_DescriptorsInit(void)
{
    uint32_t ui32Loop;

    // Initialize each of the transmit descriptors.  Note that we leave the OWN
    // bit clear here since we have not set up any transmissions yet.
    for(ui32Loop = 0; ui32Loop < NUM_TX_DESCRIPTORS; ui32Loop++)
    {
		//DES0
		//control and status bits
		//Bit 31 is the ``OWN'' bit which, when set, indicates
		//that the hardware has control of the descriptor.
        g_psTxDescriptor[ui32Loop].ui32CtrlStatus =(
							DES0_TX_CTRL_LAST_SEG  |
							DES0_TX_CTRL_FIRST_SEG |
							DES0_TX_CTRL_INTERRUPT | 	//send interrupt on completion
							DES0_TX_CTRL_CHAINED   |	//DES3 contains next descriptor address
//							DES0_TX_CTRL_IP_ALL_CKHSUMS);//insert CRC
							DES0_TX_CTRL_NO_CHKSUM);	//don't insert CRC

        //DES1
		//size of the buffer attached to the descriptor and various control bits
        g_psTxDescriptor[ui32Loop].ui32Count =
							(DES1_TX_CTRL_SADDR_INSERT |
							(TX_BUFFER_SIZE << DES1_TX_CTRL_BUFF1_SIZE_S));
		//DES2
		//pointer to the buffer containing the data to transmit
        g_psTxDescriptor[ui32Loop].pvBuffer1  = g_pui8TxBuffer;
		//DES3
		//set the next descriptor address
        g_psTxDescriptor[ui32Loop].DES3.pLink =
							(ui32Loop == (NUM_TX_DESCRIPTORS - 1)) ?
							g_psTxDescriptor : &g_psTxDescriptor[ui32Loop + 1];
    }

    // Initialize each of the receive descriptors.  We clear the OWN bit here
    // to make sure that the receiver doesn't start writing anything
    // immediately.
    for(ui32Loop = 0; ui32Loop < NUM_RX_DESCRIPTORS; ui32Loop++)
    {
		//DES0
		//control and status bits
		//Bit 31 is the ``OWN'' bit which, when set, indicates
		//that the hardware has control of the descriptor.
        g_psRxDescriptor[ui32Loop].ui32CtrlStatus = 0;
        //DES1
		//size of the buffer attached to the descriptor and various control bits
        g_psRxDescriptor[ui32Loop].ui32Count =
							(DES1_RX_CTRL_CHAINED |
							(RX_BUFFER_SIZE << DES1_RX_CTRL_BUFF1_SIZE_S));
		//DES2
		//pointer to the buffer that will contains the data2 to receive
        g_psRxDescriptor[ui32Loop].pvBuffer1 = g_pui8RxBuffer;
		//DES3
		//set the next descriptor address
        g_psRxDescriptor[ui32Loop].DES3.pLink =
							(ui32Loop == (NUM_RX_DESCRIPTORS - 1)) ?
							g_psRxDescriptor : &g_psRxDescriptor[ui32Loop + 1];
    }

    // Set the descriptor pointers in the hardware.
    EMACRxDMADescriptorListSet(EMAC0_BASE, g_psRxDescriptor);
    EMACTxDMADescriptorListSet(EMAC0_BASE, g_psTxDescriptor);

    // Start from the beginning of both descriptor chains.  We actually set
    // the transmit descriptor index to the last descriptor in the chain
    // since it will be incremented before use and this means the first
    // transmission we perform will use the correct descriptor.
    g_ui32RxDescIndex = 0;
    g_ui32TxDescIndex = NUM_TX_DESCRIPTORS - 1;
}
/**************************************************************************
* Function     : Ethernet_Init()    //do not confuse with EthernetInit( ) in
*                                   //driverLib/etherenet.h
*
* Description  : Initialize the Ethenet dirver
* Input        :
* Output  	   :
* Revision History:
**************************************************************************/
#define ETHERNET_LINK_MAX_COUNT		2000	//wait for 2sec
uint16_t Ethernet_Init(void) {

	uint32_t tmp32;

	// Configure for use with the internal PHY.
	EMACPHYConfigSet(	EMAC0_BASE,
						EMAC_PHY_TYPE_INTERNAL	  |
						EMAC_PHY_INT_MDIX_EN	  |
						EMAC_PHY_AN_100B_T_FULL_DUPLEX);

	// Initialize the MAC and set the DMA mode.
	EMACReset(EMAC0_BASE);   // Reset the MAC.
	EMACInit(EMAC0_BASE,gSysCtlClock,
						EMAC_BCONFIG_MIXED_BURST |
						EMAC_BCONFIG_PRIORITY_FIXED,
						4,
						4,
						0);
	EMACConfigSet(EMAC0_BASE,
					  (	EMAC_CONFIG_FULL_DUPLEX		    | 	//operate in full- or half-duplex mode
						EMAC_CONFIG_CHECKSUM_OFFLOAD	|	//enables header checksum checking
						EMAC_CONFIG_7BYTE_PREAMBLE	    |	//number of bytes of preamble added
															//to the beginning of every transmitted
															//frame
						EMAC_CONFIG_IF_GAP_96BITS	    |	//interframe gap between transmitted
															//frames
						EMAC_CONFIG_USE_MACADDR0		|	//which MAC address is used during
															//insertion or replacement for all
															//transmitted frames
						EMAC_CONFIG_SA_FROM_DESCRIPTOR  |	//Control over insertion or replacement
															//of the source address in all
															//transmitted frames
						EMAC_CONFIG_BO_LIMIT_1024	    ),	//The back-off limit determines the
															//range of the random time that the MAC
															//delays after a collision and before
															//attempting to retransmit a frame.
					  (	EMAC_MODE_RX_STORE_FORWARD	    |	//receive DMA to read frames from
															//the FIFO only after the complete
															//frame has been written to it.
															//If this mode is enabled, the
															//receive threshold is ignored.
						EMAC_MODE_TX_STORE_FORWARD	    |	//start transmitting a frame only
															//after the whole frame has been
															//written to the transmit FIFO.
															//If this mode is enabled, the
															//transmit threshold is ignored.
						EMAC_MODE_TX_THRESHOLD_64_BYTES |	//transmit FIFO threshold
						EMAC_MODE_RX_THRESHOLD_64_BYTES ),	//receive FIFO threshold
						0);


	// Program the hardware with its MAC address (for filtering).
	EMACAddrSet(EMAC0_BASE, 0, (uint8_t *)&gEthernetMacAddr);

	// Wait for the link to become active.
 	tmp32 = 0;
	while(	((EMACPHYRead(EMAC0_BASE, 0, EPHY_BMSR) & EPHY_BMSR_LINKSTAT) == 0) &&
			(tmp32 < ETHERNET_LINK_MAX_COUNT)) {
		SysCtlDelay(gSysCtlClock * 1e-3);	//wait for 2sec
		tmp32++;
	}

	// Init buffers descriptors
	Ethernet_DescriptorsInit();

	// Set MAC filtering options.  We receive all broadcast and multicast
	// packets along with those addressed specifically for us.
	EMACFrameFilterSet(EMAC0_BASE, (EMAC_FRMFILTER_SADDR          |	//configures the MAC to drop
																	//received frames when the
																	//source address field in the
																	//frame does not match the
																	//values programmed into the
																	//enabled SA registers.
									EMAC_FRMFILTER_PASS_MULTICAST |	//configures the MAC to pass
																	//all incoming frames with
																	//multicast destinations
																	//addresses
									EMAC_FRMFILTER_PROMISCUOUS    | //operate in promiscuous mode
																	//where all received frames are
																	//passed to the application
																	//and the SA and DA filter
																	//status bits of the descriptor
																	//receive status word are always
																	//cleared
									EMAC_FRMFILTER_PASS_ALL_CTRL));	//passes all control frames

	// Clear any pending interrupts.
	EMACIntClear(EMAC0_BASE, EMACIntStatus(EMAC0_BASE, false));

	// Enable the Ethernet MAC transmitter and receiver.
	EMACTxEnable(EMAC0_BASE);
	EMACRxEnable(EMAC0_BASE);

	// Enable the Ethernet RX Packet interrupt source.
	// EMAC_INT_NORMAL_INT is the logical OR of the masked state of
	// EMAC_INT_TRANSMIT | EMAC_INT_RECEIVE | EMAC_INT_TX_NO_BUFFER |
	// EMAC_INT_EARLY_RECEIVE.
	EMACIntDisable(EMAC0_BASE, UINT32_MAX);
	EMACIntClear(  EMAC0_BASE, UINT32_MAX);
	EMACIntEnable( EMAC0_BASE, EMAC_INT_RECEIVE | EMAC_INT_NORMAL_INT);

	// Enable the Ethernet interrupt.
	IntEnable(INT_EMAC0);

	// Mark the first receive descriptor as available to the DMA to start
	// the receive processing.
	g_psRxDescriptor[g_ui32RxDescIndex].ui32CtrlStatus |= DES0_RX_CTRL_OWN;

	//-------------------------------------------------------------
	//initializer the router (mostly the router lookup table)
	//-------------------------------------------------------------
	return SUCCESS;
}
/**************************************************************************
* Function     : Ethernet_Handler()
*
* Description  : process interrupts from the Etherent driver
*                in all cases, the active packet field is used to figure
*                out which packet is beeing processed.
*
*                   if( Rx Error)
*                       flush the packet from Rx FIFO
*                   if(Tx Error)
*                       set Tx active packet to NULL so it can be retransmited
*
*                   if(Rx FIFO interrupt)
*                       disable the Rx FIFO interrupt
*                       Call the Rx FIFO task
*                           allocate memory for a new packet
*                           set active packet
*                           set DMA to read packet from Rx FIFO
*                   if(Rx DMA done & there is an active packet)
*                       enable the Rx FIFO interrupt
*						Call the Rx FIFO task
*							set Rx active packet as READY
*							clear Rx active packet field ready for a new one)
*
*                   if(Tx FIFO interrupt)
*                       if there is no current active packet
*                          set Ethernet Tx Task event to start a new transmit
*                   if(Tx DMA done)
*                       (data has been moved from RAM to Tx FIFO)
*                       Trigger data transmission from Tx FIFO
*						Call the Rx FIFO task
*							Delete the Tx active packet
*							set active Tx packet field to NULL (ready for a new one)
*
* Input        :
* Output  	   :
* Revision History:
*       khaled Saab     Nov 21th 2010   creation date
**************************************************************************/
void Ethernet_Handler(void) {

	uint32_t status;

	//----------------------------------------------------------------
	// Read and Clear the  masked interrupt status.
	//----------------------------------------------------------------
    status = EMACIntStatus(EMAC0_BASE, true);
    EMACIntClear(EMAC0_BASE, status);

	//----------------------------------------------------------------
	//Check for Rx Error
	//----------------------------------------------------------------
	if(status & DES0_RX_STAT_ERR) {	// RX Error
	}

	//----------------------------------------------------------------
	//Check for Tx Error
	//----------------------------------------------------------------
	if(status & DES0_TX_STAT_ERR) {	//TX Error
	}

	//----------------------------------------------------------------
	//Check for Rx interrupt
	//	read the packet (call Ethernet Rx Task)
	//----------------------------------------------------------------
	if(status & EMAC_INT_RECEIVE) {

		//start the receive task to pull the packet from the buffer
		isr_evt_set(TASK_EVENT_RX_FLAG, gTaskEthernetRx_ID);
	}//if its an Rx interrupt

	//----------------------------------------------------------------
	//Check for Tx interrupt (we are done with the previous packet Tx)
	//Nothing to do, the packet has been sent out of the Ethernet Tx
	//FIFO, and the packet has been removed from the queue.
	//This is said, there is no harm in checking if there is packets
	//in the Ethernet Tx queue, in which case, schedule the task to run.
	//----------------------------------------------------------------
	if(status & EMAC_INT_TRANSMIT) {
		if(queueLength(&gQueueEthernetTx) > 0)
			isr_evt_set(TASK_EVENT_TX_FLAG, gTaskEthernetTx_ID);
	}
}
/**************************************************************************
* Function     : EthernetPacketProcess
*
* Description  : process the Ethernet packet
* Input        : packet_p  : packet to process
* Output       :
* Return       : SUCCESS 	The packet has been processed successfully
*							(either the packet was processed locally by
*							this core, or it has been sent to the DSP
*							for processing). In all cases, the packet
*							is removed from the gQueueEthernetRx queue.
*				 FAIL		Failed to process the packet. The packet
*							is removed from the gQueueEthernetRx queue.
*				 BUSY		we where not able to process the packet.
*							The packet is removed from the gQueueEthernetRx
*							queue.
*
* Note         : this function will EMOVE the packet from the
*				 gQueueEthernetRx queue.
* Revision     :
*	Khaled Saab 	Dec 4th 2010    	creation date.
*	Khaled Saab		August 12th 2011	added ARP processing.
*	Khaled Saab		April 24th 2012		split the EthernetPacketProcess( )
*										and added EthernetProcessIp( ).
*										added BUSY return.
**************************************************************************/
uint16_t EthernetPacketProcess(PACKET_HEADER *packet_p) {


    uint16_t status = FAIL;
	struct uip_eth_hdr *ethernetHeader_p;	//Stellaris header (dst, src, and type)

	//---------------------------------------------
	//initialization
	//---------------------------------------------
	ethernetHeader_p   = (struct uip_eth_hdr *) packetGetPtr(packet_p);

	//---------------------------------------------
	//process ARP packets
	//---------------------------------------------
	if(ethernetHeader_p->type == HTONS(UIP_ETHTYPE_ARP)) {
		status = EthernetProcessARP(packet_p);
	}

	//---------------------------------------------
	//processs Ip packets
	//---------------------------------------------
	else if(ethernetHeader_p->type == HTONS(UIP_ETHTYPE_IP)) {
		status = EthernetProcessIp(packet_p);
	}

	//---------------------------------------------
	//if we got this far, its not an ARP nor an IP message
	//  remove it
	//---------------------------------------------
	if(status != SUCCESS) {
		queueRemove(packet_p);
		packetRemove(packet_p);
	}

    return status;    //SUCCESS/FAIL/BUSY
}
/**************************************************************************
* Function     : EthernetProcessIp()
*
* Description  : Reformat the message (update the dest, src, and message length).
*
*				 Update the routing table.
*				 If the destination Ip address is in the router lookup table
*				 	get the power line destination address from the router
*				 	loopkup table.
*				 else
*					store the new address in the router loopup table, and
*					compute the power line address. In the future, we should
*					replace it by a power line ARP request i.e. broadcast
*					an ARP request for that specific ip address.
*
* Input        : packet_p	: pointer to the IP message
*							  The packet could be modified.
* Output  	   :
* return       : SUCCES if the IP message was processed successfully.
*				 FAIL	if there was some error processing the message.
*
* Revision History:
*	Khaled Saab April 24th 2012	creation date (it was split form the
*								EthernetPacketProcess function above)
*	Khaled Saab	Aug.   8th 2012	added icmp message handling (ping)
**************************************************************************/
uint16_t EthernetProcessIp(PACKET_HEADER *packet_p) {

	uint16_t status;

	struct ethip_hdr    *ipHeader_p;			//ethernet Ip header
	struct icmp_payload *icmpHeader_p;			//icmp header

	PACKET_HEADER *reply_p = NULL;
	struct ethip_hdr    *replyIpHeader_p;		//ethernet Ip header
	struct icmp_payload *replyIcmpHeader_p;		//icmp header

	//---------------------------------------------
	//sanity check
	//---------------------------------------------
	if(packet_p == NULL)
		return FAIL;

	//---------------------------------------------
	//initialization
	//---------------------------------------------
	ipHeader_p = (struct ethip_hdr *)   packetGetPtr(packet_p);

	//---------------------------------------------
	//check if its an ICMP packet destined to us
	//---------------------------------------------
	if(	(ipHeader_p->proto == UIP_PROTO_ICMP) &&
		(uip_ipaddr_cmp(ipHeader_p->destipaddr, gEthernetIpAddr))) {

		//if its an echo request
		icmpHeader_p = (struct icmp_payload *)	packetPtr(packet_p, sizeof(struct ethip_hdr));
		if(icmpHeader_p->type == ICMP_ECHO) {

			//We generate an answer by
			//	switching the dest and src ip addresses,
			// 	setting the icmp type to ECHO_RESPONSE and updating the icmp header checksum.
			//	Set the correct TTL and recalculate the header checksum
			reply_p = packetAcquire(packetLength(packet_p));
			if(reply_p == NULL)	return BUSY;

			//set the reply ip header
			replyIpHeader_p   = (struct ethip_hdr *) packetGetPtr(reply_p);
			replyIcmpHeader_p = (struct icmp_payload *)	packetPtr(reply_p, sizeof(struct ethip_hdr));

			memcpy(replyIpHeader_p, ipHeader_p, packetLength(packet_p));
			memcpy(replyIpHeader_p->ethhdr.src.addr , gEthernetMacAddr           , 6);
			memcpy(replyIpHeader_p->ethhdr.dest.addr, ipHeader_p->ethhdr.src.addr, 6);
			memcpy(replyIpHeader_p->srcipaddr       , gEthernetIpAddr            , 4);
			memcpy(replyIpHeader_p->destipaddr      , ipHeader_p->srcipaddr      , 4);

			// adjust the icmp header checksum (from echo request to echo reply)
			if (replyIcmpHeader_p->chksum >= htons(0xffff - (ICMP_ECHO << 8))) {
				replyIcmpHeader_p->chksum += htons(ICMP_ECHO << 8) + 1;
			} else {
				replyIcmpHeader_p->chksum += htons(ICMP_ECHO << 8);
			}

			//compute the ethernet checksum
			replyIcmpHeader_p->type   = ICMP_ER;		//echo reply
			replyIpHeader_p->ttl      = IP_DEFAULT_TTL;
			replyIpHeader_p->ipchksum = 0;
			replyIpHeader_p->ipchksum = inet_chksum((void*)((uint32_t)replyIpHeader_p + sizeof(struct uip_eth_hdr)),
													sizeof(struct ip_hdr));

			//complete the packet setup
			packetLength(reply_p) = packetLength(packet_p);
			packetCmd(   reply_p) = CMD_REPLY_PACKET;
			packetSrcEP( reply_p) = ENDPOINT_CORTEX_CORE;
			packetDstEP( reply_p) = packetSrcEP(packet_p);
			packetState( reply_p)|= PACKET_READY_STATE;
		}
	}

	//---------------------------------------------
	//send the ip message to its destination.
	//no processing is done at this stage.
	//---------------------------------------------
	if(reply_p != NULL) {
		status = packetRoute(reply_p);
		
		//if we fail to send the reply message
		//	remove the reply message
		//	return FAIL so that the upper layer can decide what to do
		//	with the original request
		if(status != SUCCESS) {
			packetRemove(reply_p);
		}
		//if we succeed in sending the reply message,
		//	remove the request (it has been processed and not needed anymore)
		else {
			queueRemove(packet_p);
			packetRemove(packet_p);
		}
	}
	//there is no reply message, forward the original message to
	//its destination
	//if the message was forwarded successfully
	//	return SUCCESS (the upper layer should not delete the message)
	//else
	//	return FAIL so that the upper layer can decide what to do
	//	with the original request
	else
		status = packetRoute(packet_p);

	return status;
}
/**************************************************************************
* Function     : EthernetProcessARP()
*
* Description  : reply to an ARP request.
*				 If the ARP dst is this modem, generate an ARP reply and
*				 forward it to the proper queue.
*
* Input        : request_p	: pointer to the ARP request message
* Output  	   :
* return       : SUCCES if the ARP message was ment for us and it was
*						processed successfully (a reply was successfully
*						generated) or if its an ARP message that was not
*						ment for us but was processed successfully (moved
*						to the DSP for processing).
*				 FAIL	if the ARP message is not ment for us or there
*						was some error processing the message.
*				 BUSY 	if the ARP message was ment for us, but we where
*						unalbe to process it (not enough memory or
*						buffer exhausted).
* Revision History:
*	Khaled Saab August 12th 2011    creation date
*	Khaled Saab	April  24th 2012	added BUSY as a possible return value.
*	Khaled Saab	Aug.    8th 2012	changed the code to move all arp messages
*									that don't have this modem as their
*									destination to the DSP.
**************************************************************************/
uint16_t EthernetProcessARP(PACKET_HEADER *request_p) {

	uint16_t status;
	PACKET_HEADER *reply_p = NULL;

	struct arp_hdr *ArpRequestHeader_p;	//ARP request header
	struct arp_hdr *ArpReplyHeader_p;	//ARP reply header

	//---------------------------------------------
	//initialization
	//---------------------------------------------
	ArpRequestHeader_p = (struct arp_hdr *) packetGetPtr(request_p);

	//---------------------------------------------
	//ARP request.
	//If it asked for our address, we send out a reply.
	//---------------------------------------------
	if(	(ArpRequestHeader_p->opcode == HTONS(ARP_REQUEST)) &&
		(uip_ipaddr_cmp(ArpRequestHeader_p->dipaddr, gEthernetIpAddr))) {

		//allocate memory for the reply message
		//If the buffer is not aligned on an odd half-word then
		//it cannot use DMA. This is because the two packet length
		//bytes are written in front of the packet, and the packet
		//data must have two bytes that can be pulled off to
		//become a word and leave the remainder of the buffer
		//word aligned.
		reply_p = packetAcquire(packetLength(request_p));
		if(reply_p == NULL)	return BUSY;

		//Ethernet header
		ArpReplyHeader_p = (struct arp_hdr *) packetGetPtr(reply_p);
		memcpy(ArpReplyHeader_p->ethhdr.src.addr, gEthernetMacAddr, 6);
		memcpy(ArpReplyHeader_p->ethhdr.dest.addr, ArpRequestHeader_p->shwaddr.addr, 6);
		ArpReplyHeader_p->ethhdr.type = HTONS(UIP_ETHTYPE_ARP);

		//ARP message reply
		ArpReplyHeader_p->hwtype  = ArpRequestHeader_p->hwtype;		//ethernet
		ArpReplyHeader_p->protocol= ArpRequestHeader_p->protocol;	//ARP
		ArpReplyHeader_p->hwlen   = ArpRequestHeader_p->hwlen;		//6
		ArpReplyHeader_p->protolen= ArpRequestHeader_p->protolen;	//4
		ArpReplyHeader_p->opcode  = HTONS(ARP_REPLY);				//2

		memcpy(ArpReplyHeader_p->shwaddr.addr, gEthernetMacAddr, 6);
		memcpy(ArpReplyHeader_p->sipaddr     , gEthernetIpAddr,  4);
		memcpy(ArpReplyHeader_p->dhwaddr.addr, ArpRequestHeader_p->shwaddr.addr, 6);
		memcpy(ArpReplyHeader_p->dipaddr     , ArpRequestHeader_p->sipaddr,  4);

		//complete the packet setup
		packetLength(reply_p) = packetLength(request_p);
		packetCmd   (reply_p) = CMD_REPLY_PACKET;
		packetSrcEP (reply_p) = ENDPOINT_CORTEX_CORE;
		packetDstEP (reply_p) = packetSrcEP(request_p);	//ENDPOINT_ETHERNET
		packetState (reply_p)|= PACKET_READY_STATE;
	}//if its an ARP request with matching destination address

// 	//---------------------------------------------
// 	//ARP reply.
// 	//do nothing for now
// 	//---------------------------------------------
// 	if((ArpRequestHeader_p->opcode) == HTONS(ARP_REPLY)) {
// 	}
	//---------------------------------------------
	//its not an ARP request that have this modem as a destination
	//(ARP request for a different address, or an ARP reply)
	//	forward them to the power line.
	//---------------------------------------------
	if(reply_p != NULL) {
		status = packetRoute(reply_p);
		
		//if we fail to send the reply message
		//	remove the reply message
		//	return FAIL so that the upper layer can decide what to do
		//	with the original request
		if(status != SUCCESS) {
			packetRemove(reply_p);
		}
		//if we succeed in sending the reply message,
		//	remove the request (it has been processed and not needed anymore)
		else {
			queueRemove(request_p);
			packetRemove(request_p);
		}
	}
	//there is no reply message, forward the original message to its
	//destination
	//if the message was forwarded successfully
	//	return SUCCESS (the upper layer should not delete the message)
	//else
	//	return FAIL so that the upper layer can decide what to do
	//	with the original request
	else
		status = packetRoute(request_p);
	return status;
}
