/****************************************************************
 *
 *    Proprietary Data of Adapitve Networks Inc.
 *              All rights reserved
 *
 * Notice:
 *		This file contains trade secret data which
 *		is the property of Adpative Networks Inc.  This
 *		file may not be used for any purpose unless
 *		permission is granted in writing by an officier
 *		of Adapitve Networks Inc.
 *
 *
 * Filename
 *		Ethernet_Driver.h
 *
 * Description: Ethernet Driver rootines (read and write to Ethernet).
 *
 * Revision History:
 *		John Houle	Jan 19th 2010	original version
 *      Khaled Saab Dec  2th 2010   made it compatible with uDMA
 *                                  read and write
 *                                  removed all code related to
 *                                  http, tcp... Should be restored
 ****************************************************************/
#ifndef _ETHERNET_DRIVER_H__
#define _ETHERNET_DRIVER_H__

#ifndef __TI_COMPILER_VERSION__
#include <stdbool.h>
#endif

#include <stdint.h>

#ifndef __TI_COMPILER_VERSION__
#include <driverlib/emac.h>
#endif

#include "../StellarisWare/uip-1.0/uip.h"
#include "../../Shared/MacLink.h"
//-------------------------------------------------------------------------
// Ethernet
//-------------------------------------------------------------------------
#define ETHERNET_IP1	1		//default
#define ETHERNET_IP2	2		//KS 1
#define ETHERNET_IP3	3		//KS 2
#define ETHERNET_IP4	4		//David
#define ETHENET_IP 	ETHERNET_IP4

#if(ETHENET_IP == ETHERNET_IP1)		//default
#define DEFAULT_IPADDR0 		192
#define DEFAULT_IPADDR1 		168
#define DEFAULT_IPADDR2 		15
#define DEFAULT_IPADDR3 		5
#elif(ETHENET_IP == ETHERNET_IP2)	//KS 1
#define DEFAULT_IPADDR0 		192
#define DEFAULT_IPADDR1 		168
#define DEFAULT_IPADDR2 		119
#define DEFAULT_IPADDR3 		205
#elif(ETHENET_IP == ETHERNET_IP3)	//KS 2
#define DEFAULT_IPADDR0 		192
#define DEFAULT_IPADDR1 		254
#define DEFAULT_IPADDR2 		15
#define DEFAULT_IPADDR3 		5
#elif(ETHENET_IP == ETHERNET_IP4)	//David
#define DEFAULT_IPADDR0 		169
#define DEFAULT_IPADDR1 		254
#define DEFAULT_IPADDR2 		15
#define DEFAULT_IPADDR3 		5
#endif

#define DEFAULT_NETMASK0        255
#define DEFAULT_NETMASK1        255
#define DEFAULT_NETMASK2        255
#define DEFAULT_NETMASK3        0

#define DEFAULT_GATEWAY0        169
#define DEFAULT_GATEWAY1		254
#define DEFAULT_GATEWAY2		0
#define DEFAULT_GATEWAY3		1

#if((ETHENET_IP == ETHERNET_IP1) || (ETHENET_IP == ETHERNET_IP2) || (ETHENET_IP == ETHERNET_IP4))
#define MAC_ADDRESS1   			0x00
#define MAC_ADDRESS2   			0x70
#define MAC_ADDRESS3   			0x33
#define MAC_ADDRESS4   			0x22
#define MAC_ADDRESS5   			0xEE
#define MAC_ADDRESS6   			0x11
#elif(ETHENET_IP == ETHERNET_IP3)	//KS 2
#define MAC_ADDRESS1   			0x00
#define MAC_ADDRESS2   			0x70
#define MAC_ADDRESS3   			0x33
#define MAC_ADDRESS4   			0x22
#define MAC_ADDRESS5   			0xEE
#define MAC_ADDRESS6   			0x10
#endif

#define USE_STATIC_IP   		1	//1: DHCP is not used
#define ENABLE_10BASET  		0	//1: run the Ethernet driver in 10baseT mode

#define TCP_BUFF_SIZE 			200

//IP_DEFAULT_TTL: Default value for Time-To-Live used by transport layers.
#ifndef IP_DEFAULT_TTL
#define IP_DEFAULT_TTL   		255
#endif
//ICMP_TTL: Default value for Time-To-Live used by ICMP packets.
//XXXX dont use it, use the IP_DEFAULT_TTL directly
#ifndef ICMP_TTL
#define ICMP_TTL				(IP_DEFAULT_TTL)
#endif

//-------------------------------------------------------------------------
//
// Helper Macros for Ethernet Processing
//
//-------------------------------------------------------------------------
//
// htonl/ntohl - big endian/little endian byte swapping macros for
// 32-bit (long) values
//
//-------------------------------------------------------------------------
#ifndef htonl
    #define htonl(a)                    \
        ((((a) >> 24) & 0x000000ff) |   \
         (((a) >>  8) & 0x0000ff00) |   \
         (((a) <<  8) & 0x00ff0000) |   \
         (((a) << 24) & 0xff000000))
#endif

#ifndef ntohl
    #define ntohl(a)    htonl((a))
#endif

//-------------------------------------------------------------------------
//
// htons/ntohs - big endian/little endian byte swapping macros for
// 16-bit (short) values
//
//-------------------------------------------------------------------------
#ifndef htons
    #define htons(a)                \
        ((((a) >> 8) & 0x00ff) |    \
         (((a) << 8) & 0xff00))
#endif

#ifndef ntohs
    #define ntohs(a)    htons((a))
#endif
//-------------------------------------------------------------------------
//copied from stellarisware/uip_arp.c
//-------------------------------------------------------------------------
// //The Ethernet header.
// struct uip_eth_hdr {
//   struct uip_eth_addr dest;
//   struct uip_eth_addr src;
//   u16_t type;
// };

//DO NOT USE
//check the "struct uip_icmpip_hdr" in uip.h
//This is the same as ethip_hdr below (the only differrence
//are the names)
struct ip_hdr {
	uint8_t vhl;			// version / header length
  	uint8_t tos;			// type of service
	uint8_t len[2];			// total length
	uint8_t ipid[2];		// identification
	uint8_t ipoffset[2];	// fragment offset field
	uint8_t ttl;			// time to live
	uint8_t proto;			// protocol
	uint16_t ipchksum;		// checksum
	uint16_t srcipaddr[2];	// source IP addresses
	uint16_t destipaddr[2];	// destination IP addresses
};

//ARP header
//ethernet header | arp header
struct arp_hdr {
	//ethernet header
	struct uip_eth_hdr ethhdr;   	//ethernet header

	//arp header
	uint16_t hwtype;	   			//Hardware type
	uint16_t protocol;				//Protocol type
	uint8_t  hwlen;					//Hardware address length
	uint8_t  protolen;				//Protocol address length
	uint16_t opcode;				//ARP opcode
	struct uip_eth_addr shwaddr;	//Sender hardware address
	uint16_t sipaddr[2];			//Sender protocol address
	struct uip_eth_addr dhwaddr;	//Target hardware address
	uint16_t dipaddr[2];			//Target protocol address
};

//Ip header
//ethernet header | Ip header
struct ethip_hdr {
	//ethernet header
	struct uip_eth_hdr ethhdr;

	//IP header
	uint8_t vhl;			// version / header length
  	uint8_t tos;			// type of service
	uint8_t len[2];			// total length
	uint8_t ipid[2];		// identification
	uint8_t ipoffset[2];	// fragment offset field
	uint8_t ttl;			// time to live
	uint8_t proto;			// protocol
	uint16_t ipchksum;		// checksum
	uint16_t srcipaddr[2];	// source IP addresses
	uint16_t destipaddr[2];	// destination IP addresses
};
//tcp header
//ethernet header | Ip header | tcp header
struct tcp_hdr {
	uint16_t srcport;		//The port number of the sender.
    uint16_t destport;		//The port this packet is addressed to
	uint8_t  seqno[4];
    uint8_t  ackno[4];
    uint8_t  tcpoffset;
    uint8_t  flags;
    uint8_t  wnd[2];
	uint16_t tcpchksum;		//checksum
	uint8_t  urgp[2];
	uint8_t  optdata[4];
};


//icmp header
//ethernet header | Ip header | icmp header
struct icmp_payload {
	uint8_t type;			//echo request(8) or echo reply (0)
	uint8_t code;
	uint16_t chksum;
	uint16_t id;
	uint16_t seqno;
};

//UDP headers
//ethernet header| Ip header | upd header
struct udp_hdr {
	uint16_t srcport;		//The port number of the sender.
	uint16_t destport;		//The port this packet is addressed to
	uint16_t udplen;		//The length in bytes of the UDP header and
							//the encapsulated data.
							//The minimum value for this field is 8.
	uint16_t udpchksum;		//checksum
};

#define PACKET_GET_ETHERNET_DSTADDR(ptr)  (uint8_t *)(ptr)
#define PACKET_GET_ETHERNET_SRCADDR(ptr)  ((uint8_t *)(ptr)+6)
#define PACKET_GET_ETHERNET_TYPE(ptr)     ((uint8_t *)(ptr)+12)

#define PACKET_GET_IP_LENGTH(ptr)         ((uint8_t *)(ptr)+14+2)
#define PACKET_GET_IP_ID(ptr)             ((uint8_t *)(ptr)+14+4)
#define PACKET_GET_IP_PROTOCOL(ptr) 	  ((uint8_t *)(ptr)+14+9)
#define PACKET_GET_IP_CHECKSUM(ptr) 	  ((uint8_t *)(ptr)+14+10)
#define PACKET_GET_IP_SRCADDR(ptr) 	   	  ((uint8_t *)(ptr)+14+12)
#define PACKET_GET_IP_DSTADDR(ptr) 		  ((uint8_t *)(ptr)+14+16)

#define PACKET_GET_UDP_SRCPORT(ptr) 	  ((uint8_t *)(ptr)+14+20)
#define PACKET_GET_UDP_DSTPORT(ptr) 	  ((uint8_t *)(ptr)+14+20+2)
#define PACKET_GET_UDP_LENGTH(ptr) 		  ((uint8_t *)(ptr)+14+20+4)
#define PACKET_GET_UDP_CHECKSUM(ptr) 	  ((uint8_t *)(ptr)+14+20+6)

#define PACKET_GET_UDP_PAYLOAD(ptr) 	  ((uint8_t *)(ptr)+14+20+8)

#ifndef UIP_ETHTYPE_ARP
#define UIP_ETHTYPE_ARP 0x0806
#endif
#ifndef UIP_ETHTYPE_IP
#define UIP_ETHTYPE_IP  0x0800
#endif
#ifndef UIP_ETHTYPE_IP6
#define UIP_ETHTYPE_IP6 0x86dd
#endif

#ifndef UIP_PROTO_ICMP
#define UIP_PROTO_ICMP  1
#endif
#ifndef UIP_PROTO_TCP
#define UIP_PROTO_TCP   6
#endif
#ifndef UIP_PROTO_UDP
#define UIP_PROTO_UDP   17
#endif
#ifndef UIP_PROTO_ICMP6
#define UIP_PROTO_ICMP6 58
#endif

//ARP messeages type arp_hdr->opcode
#ifndef ARP_REQUEST
#define ARP_REQUEST 1	//ARP request
#endif
#ifndef ARP_REPLY
#define ARP_REPLY   2	//ARP reply
#endif

//icmp struct icmp_payload->type
#define ICMP_ER   0    // echo reply (ping)
#define ICMP_DUR  3    // destination unreachable
#define ICMP_SQ   4    // source quench
#define ICMP_RD   5    // redirect
#define ICMP_ECHO 8    // echo (ping)
#define ICMP_TE   11   // time exceeded
#define ICMP_PP   12   // parameter problem
#define ICMP_TS   13   // timestamp
#define ICMP_TSR  14   // timestamp reply
#define ICMP_IRQ  15   // information request
#define ICMP_IR   16   // information reply

//The listen and send port of the DHCP agent.
#ifndef DHCPC_SERVER_PORT
#define DHCPC_SERVER_PORT  67
#endif
#ifndef DHCPC_CLIENT_PORT
#define DHCPC_CLIENT_PORT  68
#endif

//The listen port of the SNMP agent. Clients have to make their requests to
//this port. Most standard clients won't work if you change this!
#ifndef SNMP_IN_PORT
#define SNMP_IN_PORT 161
#endif
//The remote port that the SNMP agent sends traps to. Most standard trap
//sinks won't work if you change this!
#ifndef SNMP_TRAP_PORT
#define SNMP_TRAP_PORT 162
#endif

//The listen port for HTTP messages
#ifndef HTTP_SERVER_PORT
#define HTTP_SERVER_PORT 80
#endif

//-------------------------------------------------------------------------
// Global varialbes redeclaration
//-------------------------------------------------------------------------
extern uint8_t gEthernetMacAddr[6];
extern uint8_t gEthernetIpAddr[4];
extern uint8_t gEthernetGateway[4];
extern uint8_t gEthernetMask[4];

//extern uint16_t gWdgEthStallCount;

// Ethernet DMA descriptors.
#ifndef __TI_COMPILER_VERSION__
#define NUM_TX_DESCRIPTORS 3
#define NUM_RX_DESCRIPTORS 3
extern tEMACDMADescriptor g_psRxDescriptor[NUM_TX_DESCRIPTORS];
extern tEMACDMADescriptor g_psTxDescriptor[NUM_RX_DESCRIPTORS];

extern uint32_t g_ui32RxDescIndex;
extern uint32_t g_ui32TxDescIndex;
#endif

// Transmit and receive buffers.
#ifndef __TI_COMPILER_VERSION__
#define RX_BUFFER_SIZE 1536
#define TX_BUFFER_SIZE 1536
extern uint8_t g_pui8RxBuffer[RX_BUFFER_SIZE];
extern uint8_t g_pui8TxBuffer[TX_BUFFER_SIZE];
#endif

//-------------------------------------------------------------------------
// Function define
//-------------------------------------------------------------------------
void Ethernet_AddressInit(      void);
uint16_t Ethernet_Init(			void);
void     Ethernet_IRQHandler(	void);
uint16_t EthernetPacketProcess(	PACKET_HEADER *packet_p);

uint16_t EthernetProcessIp(		PACKET_HEADER *packet_p);
uint16_t EthernetProcessARP(	PACKET_HEADER *request_p);
uint16_t EthernetRoutePacket(	PACKET_HEADER 		*packet_p,
								uint16_t 			*sipAddr_p,
								struct uip_eth_addr	*shwAddr_p,
								uint16_t 			*dipAddr_p,
								struct uip_eth_addr	*dhwAddr_p);
#endif //_ETHERNET_DRIVER_H__
