/****************************************************************
  *
  * Proprietary Data of Adaptive Networks, Inc.
  *              All rights reserved
  *
  * Notice:
  *        This file contains trade secret data which
  *        is the property of Adaptive Networks, Inc.  This
  *        file may not be used for any purpose unless
  *        permission is granted in writing by an officer
  *        of Adaptive Networks, Inc.
  *
  *
  * Filename
  *        taskMgr.c
  *
  * Description: task manager C code
  *
  * Revision History:
  *	Khaled Saab Oct 21th 2010   Initial version
  *	Khaled Saab Oct 10th 2013	to improve scheduling between tasks
  *								replace os_tsk_pass by
  *								os_tsk_pass_and_set_prio
  * Khaled Saab Jan 23th 2013	Added SPI Read task
  *								split the EPI structure into two:
  *								one for the EPI and another for the
  *								SPI interface
  * Khaled Saab	Mar 15th 2016	Moved taskEthernetRx() and
  *								taskEthernetTx() to Ethernet_Task.c
  * Khaled Saab	Mar 16th 2016	Moved taskUart0Rx() and
  *								taskUart0Tx() to RS232_Task.c
  * Khaled Saab	Mar 17th 2016	Moved taskUart2Rx() and
  *								taskUart2Tx() to RS485_Task.c
  ****************************************************************/
#include <stdio.h>
#include <stdbool.h>
#include <stdint.h>             //definitions like uint32_t, ...
#include <stdlib.h>				//rand
#include <RTL.h>				//mutex
#include <string.h>				//memcpy

#include <inc/hw_memmap.h>
#include <inc/hw_types.h>
#include <inc/hw_uart.h>
#include <inc/hw_gpio.h>        //GPIO base addresses
#include <inc/hw_ssi.h>
#include <inc/hw_emac.h>
#include <driverlib/emac.h>
#include <driverlib/interrupt.h>
#include <driverlib/uart.h>
#include <driverlib/gpio.h>     //GPIO definitions
#include <driverlib/udma.h>
#include <driverlib/sysctl.h>
#include "../../Shared/DSP1000.h"
#include "../../Shared/MacLink.h"
#include "../../Shared/packetAPI.h"
#include "../../Shared/queueAPI.h"


#include "../main/main.h"			//stats
#include "../main/system_TM4C129.h"	//systemReset
#include "../interfaces/A_GPIO.h"
#include "../our_Drivers/taskMgr.h"
#include "../our_Drivers/Ethernet_Driver.h"


//-----------------------------------------------------------------
//Global variables
//-----------------------------------------------------------------
OS_TID gTaskEthernetTx_ID;      //Ethernet Tx Task ID
OS_TID gTaskEthernetRx_ID;      //Ethernet Rx Task ID


/**************************************************************************
* Function     : taskInit
*
* Description  : create and initialize all tasks.
*                   - RS232 task
*                   - RS485 task
*                   - Etherent task
*                   - EPI task
*
* Input        :
* Output       :
* Return       :
* History      :
*       khaled Saab     Oct 21th 2010 creation data
*       Khaled Saab     Dec  8th 2010 added SSI tasks
*       Khaled Saab     Jan 23th 2011 enabled interrupts
**************************************************************************/
__task void taskInit(void) {

	gTaskEthernetTx_ID = os_tsk_create(taskEthernetTx,	TASK_ETHERNET_TX_PRIORITY);
	gTaskEthernetRx_ID = os_tsk_create(taskEthernetRx,	TASK_ETHERNET_RX_PRIORITY);

	//initialize Mutex
	MUT_Init();				//Init Mutex

	//remove the task (not needed any more)
	os_tsk_delete_self();
	while(1) {};	//added to remove warning message
}
