/****************************************************************
  *
  * Proprietary Data of Adaptive Networks, Inc.
  *              All rights reserved
  *
  * Notice:
  *        This file contains trade secret data which
  *        is the property of Adaptive Networks, Inc.  This
  *        file may not be used for any purpose unless
  *        permission is granted in writing by an officer
  *        of Adaptive Networks, Inc.
  *
  *
  * Filename
  *        taskMgr.h
  *
  * Description: task manager header code
  *
  * Revision History:
  *     khaled Saab     Oct 21th 2010   Initial version
  ****************************************************************/
#ifndef __TAKS_MGR_H
#define __TAKS_MGR_H

//-------------------------------------------------------------------------
// Include header Files
//-------------------------------------------------------------------------
#include <rtl.h>
#include <stdio.h>

//-------------------------------------------------------------------------
// defines
//Note: Epi Tx task and Epi Rx task should have the same priority. If not,
//		the Epi Tx task could assert CS, but Hint ISR prevent it from
//		starting the uDMA and schedule the Epi Rx task. In this case,
//		we have a lockup (Epi Tx uDMA done will not happen).
//		BUT THIS SHOULD NEVER HAPPEN: the Epi Read and Write SHOULD be
//		atomic!!!!! Test are showing otherwise.
//-------------------------------------------------------------------------
//define the task priority with respect to each others
//RTX_Config.c/OS_TASKCNT should be large enough to support all the tasks
#define TASK_ETHERNET_TX_PRIORITY       3
#define TASK_ETHERNET_RX_PRIORITY       3

//define the task timeout after which the function must
//return even if none of the events have occurred.
//You can use any value of timeout with the exception of
//0xFFFF, which you can use to specify an indefinite timeout.
//The unit of measure of the timeout argument is the number
//of system intervals.
//check RTX_Config.c for OS_CLOCK and OS_TICK configuration
#define TASK_ETHERNET_TX_TIMEOUT    0x03FF
#define TASK_ETHERNET_RX_TIMEOUT    0xFFFF

//each task has events flags that can respond to
//two main flags are used, Tx and Rx flags
#define TASK_EVENT_TX_FLAG      	0x1000	//Tx event
#define TASK_EVENT_RX_FLAG      	0x2000	//Rx event
#define TASK_EVENT_TX_ERROR_FLAG	0x4000	//Tx error event
#define TASK_EVENT_RX_ERROR_FLAG	0x8000	//Rx error event
#define TASK_EVENT_FLAG				0xF000	//any event

//-------------------------------------------------------------------------
// global variables re-declarations
//-------------------------------------------------------------------------
//declare system identification number variables
extern OS_TID gTaskEthernetTx_ID;		//Ethernet Tx Task ID
extern OS_TID gTaskEthernetRx_ID;		//Ethernet Rx Task ID

//-------------------------------------------------------------------------
// Function define
//-------------------------------------------------------------------------
//uint8_t taskInit(void);
__task void taskInit(void);

//-------------------------------------------------------------------------
// tasks declaration
//-------------------------------------------------------------------------
__task void taskEthernetTx(void);   //Ethernet Tx task
__task void taskEthernetRx(void);   //Ethernet Rx task

#endif  //__TAKS_MGR_H
